@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2007, 2008, 2009, 2010 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -36,17 +37,13 @@
 #include <syslog.h>
 #include <string.h>
 #include <pwd.h>
-#include <ctype.h>
 
-#ifdef HAVE_PATHS_H
-# include <paths.h>
-#endif
+#include <confpaths.h>
 
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif
 
-#include <mailutils/argp.h>
 #include <mailutils/body.h>
 #include <mailutils/errno.h>
 #include <mailutils/error.h>
@@ -60,6 +57,10 @@
 #include <mailutils/mu_auth.h>
 #include <mailutils/argcv.h>
 #include <mailutils/nls.h>
+#include <mailutils/daemon.h>
+#include <mailutils/acl.h>
+#include <mailutils/server.h>
+#include <mailutils/cctype.h>
 
 #ifndef INADDR_NONE
 # define INADDR_NONE -1
@@ -77,6 +78,6 @@ extern const char *username;
 extern char hostname[];
 extern struct daemon_param daemon_param;
 
-extern void read_config (const char *config_file);
-int acl_match (struct sockaddr_in *sa_in);
 void run_user_action (FILE *tty, const char *cr, mu_message_t msg);
+void convert_config (const char *config_file, FILE *outfile);
+
