@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005, 
-   2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007, 2008, 2009, 2010
+   Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,14 +27,15 @@
 #endif
 #include <unistd.h>
 
-#include <mailutils/argp.h>
 #include <mailutils/errno.h>
 #include <mailutils/locker.h>
 #include <mailutils/nls.h>
 
+#include "mailutils/libargp.h"
+
 const char *program_version = "GNU dotlock (" PACKAGE_STRING ")";
 static char doc[] =
-N_("GNU dotlock -- lock mail spool files")
+N_("GNU dotlock -- lock mail spool files.")
 "\v"
 N_("Returns 0 on success, 3 if locking the file fails because\
  it's already locked, and 1 if some other kind of error occurred.");
@@ -43,16 +44,16 @@ static char args_doc[] = N_("FILE");
 
 static struct argp_option options[] = {
   {"unlock", 'u', NULL, 0,
-   N_("Unlock"), 0},
+   N_("unlock"), 0},
 
   {"force", 'f', N_("MINUTES"), OPTION_ARG_OPTIONAL,
-   N_("Forcibly break an existing lock older than a certain time"), 0},
+   N_("forcibly break an existing lock older than a certain time"), 0},
 
   {"retry", 'r', N_("RETRIES"), OPTION_ARG_OPTIONAL,
-   N_("Retry the lock a few times"), 0},
+   N_("retry the lock a few times"), 0},
 
   {"debug", 'd', NULL, 0,
-   N_("Print details of failure reasons to stderr"), 0},
+   N_("print details of failure reasons to stderr"), 0},
 
   {NULL, 0, NULL, 0, NULL, 0}
 };
@@ -70,16 +71,18 @@ static const char *file;
 static int unlock;
 static int flags;
 static int retries;
-static int force;
+static time_t force;
 static int debug;
 
 static error_t
 parse_opt (int key, char *arg, struct argp_state *state)
 {
+  static mu_list_t lst;
+
   switch (key)
     {
     case 'd':
-      debug = 1;
+      mu_argp_node_list_new (lst, "debug", "yes");
       break;
 
     case 'u':
@@ -88,40 +91,58 @@ parse_opt (int key, char *arg, struct argp_state *state)
 
     case 'r':
       if (arg)
-	{
-	  retries = atoi (arg);
-	  if (retries <= 0)
-	    argp_error (state, _("RETRIES must be greater than 0"));
-	}
-      flags |= MU_LOCKER_RETRY;
+	mu_argp_node_list_new (lst, "retry", arg);
       break;
 
     case 'f':
-      if (arg)
-	{
-	  force = atoi (arg);
-	  if (force <= 0)
-	    argp_error (state, _("MINUTES must be greater than 0"));
-	  force *= 60;
-	}
-      flags |= MU_LOCKER_TIME;
+      mu_argp_node_list_new (lst, "force", arg ? arg : "0");
       break;
 
     case ARGP_KEY_ARG:
       if (file)
-	argp_error (state, _("Only one FILE can be specified"));
+	argp_error (state, _("only one FILE can be specified"));
       file = arg;
       break;
 
     case ARGP_KEY_NO_ARGS:
-      argp_error (state, _("FILE must be specified"));
-
+      if (!mu_help_config_mode)
+	argp_error (state, _("FILE must be specified"));
+      return ARGP_ERR_UNKNOWN;
+      
+    case ARGP_KEY_INIT:
+      mu_argp_node_list_init (&lst);
+      break;
+      
+    case ARGP_KEY_FINI:
+      mu_argp_node_list_finish (lst, NULL, NULL);
+      break;
+      
     default:
       return ARGP_ERR_UNKNOWN;
     }
   return 0;
 }
 
+
+struct mu_cfg_param dotlock_cfg_param[] = {
+  { "force", mu_cfg_time, &force, 0, NULL,
+    N_("Forcibly break an existing lock older than the specified time.") },
+  { "retry", mu_cfg_int, &retries, 0, NULL,
+    N_("Number of times to retry acquiring the lock.") },
+  { "debug", mu_cfg_bool, &debug, 0, NULL,
+    N_("Print details of failure reasons to stderr.") },
+  { NULL }
+};
+
+
+
+const char *dotlock_capa[] = {
+  "license",
+  "common",
+  "debug",
+  NULL
+};
+
 int
 main (int argc, char *argv[])
 {
@@ -131,26 +152,36 @@ main (int argc, char *argv[])
   pid_t mailgid = getegid ();
 
   /* Native Language Support */
-  mu_init_nls ();
+  MU_APP_INIT_NLS ();
 
   /* Drop permissions during argument parsing. */
 
   if (setegid (usergid) < 0)
     return MU_DL_EX_ERROR;
 
-  mu_argp_init (program_version, NULL);
   argp_err_exit_status = MU_DL_EX_ERROR;
-  argp_parse (&argp, argc, argv, 0, NULL, NULL);
+  
+  mu_argp_init (program_version, NULL);
+  if (mu_app_init (&argp, dotlock_capa, dotlock_cfg_param, 
+		   argc, argv, 0, NULL, NULL))
+    exit (1);
 
+  if (force)
+    {
+      force *= 60;
+      flags |= MU_LOCKER_TIME;
+    }
+
+  if (retries != 0)
+    flags |= MU_LOCKER_RETRY;
+  
   if ((err = mu_locker_create (&locker, file, flags)))
     {
       if (debug)
-	fprintf (stderr, _("Creating locker failed: %s\n"), mu_strerror (err));
+	mu_diag_funcall (MU_DIAG_ERROR, "mu_locker_create", NULL, err);
       return MU_DL_EX_ERROR;
     }
 
-  mu_locker_set_flags (locker, flags);
-
   if (force != 0)
     mu_locker_set_expire_time (locker, force);
 
@@ -170,10 +201,9 @@ main (int argc, char *argv[])
   mu_locker_destroy (&locker);
 
   if (debug && err)
-    fprintf (stderr,
-	     unlock ? _("Unlocking the file %s failed: %s\n") :
-	              _("Locking the file %s failed: %s\n"),
-	     file, mu_strerror (err));
+    mu_error (unlock ? _("unlocking the file %s failed: %s") :
+	      _("locking the file %s failed: %s"),
+	      file, mu_strerror (err));
 
   switch (err)
     {
