@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2005, 2007, 2009, 2010 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,6 +35,7 @@ void message_display_parts(mu_message_t msg, int indent);
 
 const char *from;
 const char *subject;
+const char *charset;
 int print_attachments;
 int indent_level = 4;
 
@@ -71,7 +73,23 @@ main (int argc, char **argv)
       else if (strcmp (argv[i], "-p") == 0)
         print_attachments = 1;
       else if (strcmp (argv[i], "-i") == 0)
-        indent_level = strtoul (argv[++i], NULL, 0);
+	{
+	  if (++i == argc)
+	    {
+	      mu_error ("-i requires argument");
+	      exit (1);
+	    }
+	  indent_level = strtoul (argv[i], NULL, 0);
+	}
+      else if (strcmp (argv[i], "-c") == 0)
+	{
+	  if (++i == argc)
+	    {
+	      mu_error ("-c requires argument");
+	      exit (1);
+	    }
+	  charset = argv[i];
+	}
       else
         break;
     }
@@ -159,11 +177,9 @@ void
 message_display_parts (mu_message_t msg, int indent)
 {
   int ret, j;
-  size_t nparts, nsubparts;
+  size_t nparts;
   mu_message_t part;
   mu_header_t hdr;
-  const char *type;
-  const char *encoding;
   mu_stream_t str;
   mu_body_t body;
   int offset, ismulti;
@@ -182,24 +198,34 @@ message_display_parts (mu_message_t msg, int indent)
   for (j = 1; j <= nparts; j++)
     {
       int status;
+      const char *hvalue;
+      char *type = NULL;
+      const char *encoding = "";
+
       MU_ASSERT (mu_message_get_part (msg, j, &part));
       MU_ASSERT (mu_message_get_header (part, &hdr));
-      status = mu_header_sget_value (hdr, MU_HEADER_CONTENT_TYPE, &type);
+      status = mu_header_sget_value (hdr, MU_HEADER_CONTENT_TYPE,
+				     &hvalue);
       if (status == MU_ERR_NOENT)
-	type = "";
+	/* nothing */;
       else if (status != 0)
+	mu_error ("Cannot get header value: %s", mu_strerror (status));
+      else
 	{
-	  type = "";
-	  mu_error ("Cannot get header value: %s", mu_strerror (status));
+	  status = mu_mimehdr_aget_disp (hvalue, &type);
+	  if (status)
+	    mu_error ("Cannot extract content type field: %s",
+		      mu_strerror (status));
 	}
-      printf ("%*.*sType of part %d = %s\n", indent, indent, "", j, type);
+      printf ("%*.*sType of part %d = %s\n", indent, indent, "",
+	      j, type ? type : "");
       print_message_part_sizes (part, indent);
       if (mu_header_sget_value (hdr, MU_HEADER_CONTENT_TRANSFER_ENCODING,
 				&encoding))
 	encoding = "";
       ismulti = 0;
-      if ((type[0]
-           && strncasecmp (type, "message/rfc822", strlen (type)) == 0)
+      if ((type
+           && mu_c_strcasecmp (type, "message/rfc822") == 0)
           || (mu_message_is_multipart (part, &ismulti) == 0 && ismulti))
         {
           if (!ismulti)
@@ -213,24 +239,24 @@ message_display_parts (mu_message_t msg, int indent)
           printf ("%*.*sEncapsulated message : %s\t%s\n",
                   indent, indent, "", from, subject);
           printf ("%*.*sBegin\n", indent, indent, "");
-          MU_ASSERT (mu_message_get_num_parts (part, &nsubparts));
-          message_display_parts (part, indent+indent_level);
+          message_display_parts (part, indent + indent_level);
           mu_message_destroy (&part, NULL);
         }
-      else if (type[0] == '\0'
-               || (strncasecmp (type, "text/plain", strlen ("text/plain")) ==
-                   0)
-               || (strncasecmp (type, "text/html", strlen ("text/html")) ==
-                   0))
-        {
-          printf ("%*.*sText Message\n", indent, indent, "");
+      else if (!type
+               || (mu_c_strcasecmp (type, "text/plain") == 0)
+               || (mu_c_strcasecmp (type, "text/html")) == 0)
+	{
+	  printf ("%*.*sText Message\n", indent, indent, "");
           printf ("%*.*sBegin\n", indent, indent, "");
           mu_message_get_body (part, &body);
           mu_body_get_stream (body, &str);
-          mu_filter_create (&str, str, encoding, 0, 0);
+          /* Make sure the original body stream is not closed when
+             str gets destroyed */
+          mu_filter_create (&str, str, encoding, MU_FILTER_DECODE,
+			    MU_STREAM_READ | MU_STREAM_NO_CLOSE);
           offset = 0;
-          while (mu_stream_readline (str, buf, sizeof (buf), offset, &nbytes) ==
-                 0 && nbytes)
+          while (mu_stream_readline (str, buf, sizeof (buf),
+				     offset, &nbytes) == 0 && nbytes)
             {
               printf ("%*.*s%s", indent, indent, "", buf);
               offset += nbytes;
@@ -241,20 +267,31 @@ message_display_parts (mu_message_t msg, int indent)
         {
           /* Save the attachements.  */
           char *fname = NULL;
-          mu_message_aget_attachment_name (part, &fname);
+
+          mu_message_aget_decoded_attachment_name (part, charset,
+						   &fname, NULL);
           if (fname == NULL)
             fname = mu_tempname (NULL);
 
           printf ("%*.*sAttachment - saving [%s]\n", indent, indent, "",
                   fname);
           printf ("%*.*sBegin\n", indent, indent, "");
-          /*FIXME: What is the 'data' argument for? */
-          mu_message_save_attachment (part, NULL, NULL);
+          if (charset)
+	    {
+	      mu_mime_io_buffer_t info;
+	      mu_mime_io_buffer_create (&info);
+	      mu_mime_io_buffer_set_charset (info, charset);
+	      MU_ASSERT (mu_message_save_attachment (part, NULL, info));
+	      mu_mime_io_buffer_destroy (&info);
+	    }
+	  else
+	    MU_ASSERT (mu_message_save_attachment (part, fname, NULL));
           if (print_attachments)
             print_file (fname, indent);
           free (fname);
         }
       printf ("\n%*.*sEnd\n", indent, indent, "");
+      free (type);
     }
 }
 
