@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2007, 2010 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -16,10 +17,13 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301 USA */
 
-#include <mailutils/mutil.h>
-#include <mailutils/argp.h>
-
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+#include <stdlib.h>
 #include <stdio.h>
+#include <mailutils/mutil.h>
+#include "mailutils/libargp.h"
 
 const char *capa[] = {
   "address",
@@ -31,8 +35,8 @@ main (int argc, char *argv[])
 {
   int arg = 1;
 
-  if (mu_argp_parse (NULL, &argc, &argv, 0, capa, &arg, NULL))
-    abort ();
+  if (mu_app_init (NULL, capa, NULL, argc, argv, 0, &arg, NULL))
+    exit (1);
 
   if (!argv[arg])
     printf ("current user -> %s\n", mu_get_user_email (0));
