@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008,
+   2009, 2010 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@ static int align = 0;      /* Tidy mode. -t option. */
 static int select_attribute;
 
 const char *program_version = "frm (" PACKAGE_STRING ")";
-static char doc[] = N_("GNU frm -- display From: lines");
+static char doc[] = N_("GNU frm -- display From: lines.");
 
 static struct attr_tab {
   char *name;      /* Attribute name */
