@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007, 2009, 2010 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -19,7 +20,7 @@
 #include "guimb.h"
 
 void
-util_error (char *fmt, ...)
+util_error (const char *fmt, ...)
 {
   va_list ap;
 
@@ -33,34 +34,36 @@ util_error (char *fmt, ...)
 char *
 util_get_sender (int msgno)
 {
-  header_t header = NULL;
-  address_t addr = NULL;
-  message_t msg = NULL;
-  char buffer[512];
+  mu_header_t header = NULL;
+  mu_address_t addr = NULL;
+  mu_message_t msg = NULL;
+  const char *buffer;
+  char *email;
 
-  mailbox_get_message (mbox, msgno, &msg);
-  message_get_header (msg, &header);
-  if (header_get_value (header, MU_HEADER_FROM, buffer, sizeof (buffer), NULL)
-      || address_create (&addr, buffer))
+  mu_mailbox_get_message (mbox, msgno, &msg);
+  mu_message_get_header (msg, &header);
+  if (mu_header_sget_value (header, MU_HEADER_FROM, &buffer)
+      || mu_address_create (&addr, buffer))
     {
-      envelope_t env = NULL;
-      message_get_envelope (msg, &env);
-      if (envelope_sender (env, buffer, sizeof (buffer), NULL)
-	  || address_create (&addr, buffer))
+      mu_envelope_t env = NULL;
+      mu_message_get_envelope (msg, &env);
+      
+      if (mu_envelope_sget_sender (env, &buffer)
+	  || mu_address_create (&addr, buffer))
 	{
-	  util_error (_("Cannot determine sender name (msg %d)"), msgno);
+	  util_error (_("cannot determine sender name (msg %d)"), msgno);
 	  return NULL;
 	}
     }
 
-  if (address_get_email (addr, 1, buffer, sizeof (buffer), NULL))
+  if (mu_address_aget_email (addr, 1, &email))
     {
-      util_error (_("Cannot determine sender name (msg %d)"), msgno);
-      address_destroy (&addr);
+      util_error (_("cannot determine sender name (msg %d)"), msgno);
+      mu_address_destroy (&addr);
       return NULL;
     }
 
-  address_destroy (&addr);
-  return strdup (buffer);
+  mu_address_destroy (&addr);
+  return email;
 }
 
