@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2006, 2007, 2008, 2010 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,6 +47,7 @@ auth_add (char *name, imap4d_auth_handler_fp handler)
     {
       mu_list_create (&imap_auth_list);
       mu_list_set_comparator (imap_auth_list, comp);
+      mu_list_set_destroy_item (imap_auth_list, mu_list_free_item);
     }
   mu_list_append (imap_auth_list, (void*)p);
 }
@@ -80,8 +82,7 @@ _auth_try (void *item, void *data)
 
   if (strcmp (p->name, ap->auth_type) == 0)
     {
-      ap->result = p->handler (ap->command,
-			       ap->auth_type, ap->arg, &ap->username);
+      ap->result = p->handler (ap->command, ap->auth_type, &ap->username);
       return 1;
     }
   return 0;
@@ -93,24 +94,30 @@ imap4d_auth_capability ()
   mu_list_do (imap_auth_list, _auth_capa, NULL);
 }
 
+/*
+6.2.1.  AUTHENTICATE Command
+
+   Arguments:  authentication mechanism name
+*/
+
 int
-imap4d_authenticate (struct imap4d_command *command, char *arg)
+imap4d_authenticate (struct imap4d_command *command, imap4d_tokbuf_t tok)
 {
-  char *sp = NULL;
   char *auth_type;
   struct auth_data adata;
+
+  if (imap4d_tokbuf_argc (tok) != 3)
+    return util_finish (command, RESP_BAD, "Invalid arguments");
   
-  auth_type = util_getword (arg, &sp);
-  util_unquote (&auth_type);
-  if (!auth_type)
-    return util_finish (command, RESP_BAD, "Too few arguments");
+  auth_type = imap4d_tokbuf_getarg (tok, IMAP4_ARG_1);
 
   if (tls_required)
-    return util_finish (command, RESP_NO, "Command disabled: Use STARTTLS first");
+    return util_finish (command, RESP_NO,
+			"Command disabled: Use STARTTLS first");
   
   adata.command = command;
   adata.auth_type = auth_type;
-  adata.arg = sp;
+  adata.arg = NULL;
   adata.username = NULL;
 
   if (mu_list_do (imap_auth_list, _auth_try, &adata) == 0)
