@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2004, 
-   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
+   2009, 2010 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -325,7 +325,7 @@ util_send_qstring (const char *buffer)
 int
 util_send_literal (const char *buffer)
 {
-  return util_send ("{%u}\r\n%s", strlen (buffer), buffer);
+  return util_send ("{%lu}\r\n%s", (unsigned long) strlen (buffer), buffer);
 }
 
 /* Send an unsolicited response.  */
@@ -951,7 +951,10 @@ util_register_event (int old_state, int new_state,
   evp->action = action;
   evp->data = data;
   if (!event_list)
-    mu_list_create (&event_list);
+    {
+      mu_list_create (&event_list);
+      mu_list_set_destroy_item (event_list, mu_list_free_item);
+    }
   mu_list_append (event_list, (void*)evp);
 }
 
@@ -1375,19 +1378,17 @@ imap4d_getline (char **pbuf, size_t *psize, size_t *pnbytes)
   if (rc == 0)
     {
       char *s = *pbuf;
-      len = util_trim_nl (s, len);
-      if (imap4d_transcript)
-        {
-          if (len)
-            mu_diag_output (MU_DIAG_DEBUG, "recv: %s", s);
-          else
-            mu_diag_output (MU_DIAG_DEBUG, "got EOF");
-        }
+
       if (len == 0)
         {
+	  if (imap4d_transcript)
+            mu_diag_output (MU_DIAG_DEBUG, "got EOF");
           imap4d_bye (ERR_NO_IFILE);
           /*FIXME rc = ECONNABORTED;*/
         }
+      len = util_trim_nl (s, len);
+      if (imap4d_transcript)
+	mu_diag_output (MU_DIAG_DEBUG, "recv: %s", s);
       if (pnbytes)
 	*pnbytes = len;
     }
