@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2004, 2005, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2004, 2005, 2007, 2008, 2009, 2010 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -30,14 +31,13 @@ struct mu_gsasl_module_data
 
 int mu_gsasl_module_init (enum mu_gocs_op, void *);
 
-struct mu_gsasl_module_data mu_gsasl_module_data;
+extern struct mu_gsasl_module_data mu_gsasl_module_data;
 
-#ifdef USE_GSASL
+#ifdef WITH_GSASL
 #include <gsasl.h>
 
 int mu_gsasl_stream_create (mu_stream_t *stream, mu_stream_t transport,
-			    Gsasl_session_ctx *ctx,
-			    int flags);
+			    Gsasl_session *ctx, int flags);
 
 #endif
 
