@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2006, 
-   2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2005, 2006, 2007, 2010 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -28,6 +28,8 @@
 extern "C" {
 #endif
 
+extern char *mu_ticket_file;
+  
 int mu_set_mail_directory (const char *p);
 int mu_set_mailbox_pattern (const char *pat);
 void mu_set_folder_directory (const char *p);
@@ -64,6 +66,16 @@ extern int  mu_mailbox_sync            (mu_mailbox_t);
 extern int  mu_mailbox_save_attributes (mu_mailbox_t)
                                        __attribute__ ((deprecated));
 
+#define MU_UIDL_LENGTH 70
+#define MU_UIDL_BUFFER_SIZE (MU_UIDL_LENGTH+1)
+				       
+struct mu_uidl
+{
+  size_t msgno;
+  char uidl[MU_UIDL_BUFFER_SIZE];
+};
+extern int  mu_mailbox_get_uidls       (mu_mailbox_t, mu_list_t *);
+
 /* Update and scanning.  */
 extern int  mu_mailbox_get_size        (mu_mailbox_t, mu_off_t *size);
 extern int  mu_mailbox_is_updated      (mu_mailbox_t);
@@ -95,6 +107,9 @@ extern int  mu_mailbox_get_observable  (mu_mailbox_t, mu_observable_t *);
 /* Locking */  
 extern int mu_mailbox_lock (mu_mailbox_t mbox);
 extern int mu_mailbox_unlock (mu_mailbox_t mbox);
+
+extern int mu_mailbox_get_iterator (mu_mailbox_t mbx,
+				    mu_iterator_t *piterator);
   
 #ifdef __cplusplus
 }
