@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2006, 2007,
-   2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2005, 2006, 2007, 2009, 2010 Free
+   Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -29,36 +29,11 @@ mu_scm_error (const char *func_name, int status,
 {
   scm_error_scm (scm_from_locale_symbol ("mailutils-error"),
 		 func_name ? scm_from_locale_string (func_name) : SCM_BOOL_F,
-		 scm_makfrom0str (fmt),
+		 scm_from_locale_string (fmt),
 		 args,
 		 scm_list_1 (scm_from_int (status)));
 }
 
-SCM
-mu_scm_makenum (unsigned long val)
-#ifndef HAVE_SCM_LONG2NUM
-{
-  if (SCM_FIXABLE ((long) val))
-    return scm_from_int (val);
-
-#ifdef SCM_BIGDIG
-  return scm_long2big (val);
-#else /* SCM_BIGDIG */
-  return scm_make_real ((double) val);
-#endif /* SCM_BIGDIG */
-}
-#else
-{
-  return scm_long2num (val);
-}
-#endif
-
-void
-mu_set_variable (const char *name, SCM value)
-{
-  scm_c_define (name, value);
-}
-
 SCM _mu_scm_package_string; /* STRING: PACKAGE_STRING */
 SCM _mu_scm_package;        /* STRING: PACKAGE */
 SCM _mu_scm_version;        /* STRING: VERSION */
@@ -113,8 +88,8 @@ register_format (const char *name)
 }
     
 
-SCM_DEFINE (scm_mu_register_format, "mu-register-format", 0, 0, 1,
-	    (SCM REST),
+SCM_DEFINE_PUBLIC (scm_mu_register_format, "mu-register-format", 0, 0, 1,
+		   (SCM rest),
 "Registers desired mailutils formats.  Any number of arguments can be given.\n"
 "Each argument must be one of the following strings:\n\n"
 "@multitable @columnfractions 0.3 0.6\n"
@@ -133,7 +108,7 @@ SCM_DEFINE (scm_mu_register_format, "mu-register-format", 0, 0, 1,
 {
   int status;
 
-  if (REST == SCM_EOL)
+  if (scm_is_null (rest))
     {
       status = register_format (NULL);
       if (status)
@@ -143,11 +118,14 @@ SCM_DEFINE (scm_mu_register_format, "mu-register-format", 0, 0, 1,
     }
   else
     {
-      for (; REST != SCM_EOL; REST = SCM_CDR (REST))
+      for (; !scm_is_null (rest); rest = SCM_CDR (rest))
 	{
-	  SCM scm = SCM_CAR (REST);
+	  char *s;
+	  SCM scm = SCM_CAR (rest);
 	  SCM_ASSERT (scm_is_string (scm), scm, SCM_ARGn, FUNC_NAME);
-	  status = register_format (scm_i_string_chars (scm));
+	  s = scm_to_locale_string (scm);
+	  status = register_format (s);
+	  free (scm);
 	  if (status)
 	    mu_scm_error (FUNC_NAME, status,
 			  "Cannot register format ~A",
@@ -158,14 +136,14 @@ SCM_DEFINE (scm_mu_register_format, "mu-register-format", 0, 0, 1,
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (scm_mu_strerror, "mu-strerror", 1, 0, 0,
-	    (SCM ERR),
-"Return the error message corresponding to ERR, which must be\n"
+SCM_DEFINE_PUBLIC (scm_mu_strerror, "mu-strerror", 1, 0, 0,
+		   (SCM err),
+"Return the error message corresponding to @var{err}, which must be\n"
 "an integer value.\n")
 #define FUNC_NAME s_scm_mu_strerror
 {
-  SCM_ASSERT (scm_is_integer (ERR), ERR, SCM_ARG1, FUNC_NAME);
-  return scm_makfrom0str (mu_strerror (scm_to_int (ERR)));
+  SCM_ASSERT (scm_is_integer (err), err, SCM_ARG1, FUNC_NAME);
+  return scm_from_locale_string (mu_strerror (scm_to_int (err)));
 }
 #undef FUNC_NAME
 
@@ -191,24 +169,30 @@ mu_scm_init ()
 {
   int i;
 
-  _mu_scm_mailer = scm_makfrom0str ("sendmail:" PATH_SENDMAIL);
-  mu_set_variable ("mu-mailer", _mu_scm_mailer);
+  _mu_scm_mailer = scm_from_locale_string ("sendmail:" PATH_SENDMAIL);
+  scm_c_define ("mu-mailer", _mu_scm_mailer);
 
-  _mu_scm_debug = mu_scm_makenum(0);
-  mu_set_variable ("mu-debug", _mu_scm_debug);
-
-  _mu_scm_package = scm_makfrom0str (PACKAGE);
-  mu_set_variable ("mu-package", _mu_scm_package);
-
-  _mu_scm_version = scm_makfrom0str (VERSION);
-  mu_set_variable ("mu-version", _mu_scm_version);
-
-  _mu_scm_package_string = scm_makfrom0str (PACKAGE_STRING);
-  mu_set_variable ("mu-package-string", _mu_scm_package_string);
+  _mu_scm_debug = scm_from_int (0);
+  scm_c_define ("mu-debug", _mu_scm_debug);
 
+  _mu_scm_package = scm_from_locale_string (PACKAGE);
+  scm_c_define ("mu-package", _mu_scm_package);
+  scm_c_export ("mu-package", NULL);
+  
+  _mu_scm_version = scm_from_locale_string (VERSION);
+  scm_c_define ("mu-version", _mu_scm_version);
+  scm_c_export ("mu-version", NULL);
+  
+  _mu_scm_package_string = scm_from_locale_string (PACKAGE_STRING);
+  scm_c_define ("mu-package-string", _mu_scm_package_string);
+  scm_c_export ("mu-package-string", NULL);
+  
   /* Create MU- attribute names */
   for (i = 0; attr_kw[i].name; i++)
-    scm_c_define(attr_kw[i].name, scm_from_int(attr_kw[i].value));
+    {
+      scm_c_define (attr_kw[i].name, scm_from_int(attr_kw[i].value));
+      scm_c_export (attr_kw[i].name, NULL);
+    }
   
   mu_scm_mutil_init ();
   mu_scm_mailbox_init ();
