@@ -1,10 +1,11 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2006 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2006, 2007, 2009, 2010 Free
+   Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,6 +27,7 @@
 #include <errno.h>
 #include <sys/time.h>
 #include <string.h>
+#include <confpaths.h>
 
 #include <mailutils/mailbox.h>
 #include <mailutils/message.h>
@@ -46,7 +48,7 @@
 #include <mailutils/mime.h>
 #include <mailutils/registrar.h>
 #include <mailutils/mu_auth.h>
+#include <mailutils/cstr.h>
 
 #include <mailutils/guile.h>
 
-
