@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2005, 2007, 2010 Free Software Foundation,
+   Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -50,6 +51,13 @@ struct _mu_list
   struct _mu_iterator *itr;
 };
 
+extern void _mu_list_clear (mu_list_t list);
+extern void _mu_list_insert_sublist (mu_list_t list,
+				     struct list_data *current,
+				     struct list_data *head,
+				     struct list_data *tail,
+				     size_t count,
+				     int insert_before);
 
 #ifdef __cplusplus
 }
