@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005,
-   2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007, 2008, 2009, 2010
+   Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,7 +44,17 @@ make_tmp (const char *from, mu_mailbox_t *mbox)
 int
 mda (mu_mailbox_t mbx, char *username)
 {
-  deliver (mbx, username, NULL);
+  int status;
+  mu_message_t msg;
+  
+  if ((status = mu_mailbox_get_message (mbx, 1, &msg)) != 0)
+    {
+      maidag_error (_("cannot get message from the temporary mailbox: %s"),
+		    mu_strerror (status));
+      return EX_TEMPFAIL;
+    }
+
+  deliver (msg, username, NULL);
 
   if (multiple_delivery)
     exit_code = EX_OK;
@@ -62,19 +72,6 @@ maidag_stdio_delivery (int argc, char **argv)
   if (multiple_delivery)
     multiple_delivery = argc > 1;
 
-#ifdef WITH_GUILE
-  if (progfile_pattern)
-    {
-      struct mda_data mda_data;
-      
-      memset (&mda_data, 0, sizeof mda_data);
-      mda_data.mbox = mbox;
-      mda_data.argv = argv;
-      mda_data.progfile_pattern = progfile_pattern;
-      return prog_mda (&mda_data);
-    }
-#endif
-  
   for (; *argv; argv++)
     mda (mbox, *argv);
   return exit_code;
@@ -82,12 +79,12 @@ maidag_stdio_delivery (int argc, char **argv)
 
 static int biff_fd = -1;
 static struct sockaddr_in biff_in;
-static char *biff_user_name;
+static const char *biff_user_name;
 
 static int
 notify_action (mu_observer_t obs, size_t type, void *data, void *action_data)
 {
-  if (type == MU_EVT_MESSAGE_APPEND)
+  if (type == MU_EVT_MESSAGE_APPEND && biff_user_name)
     {
       mu_message_qid_t qid = data;
       mu_mailbox_t mbox = mu_observer_get_owner (obs);
@@ -143,8 +140,8 @@ attach_notify (mu_mailbox_t mbox)
 }  
 
 int
-deliver_to_user (mu_mailbox_t imbx, mu_mailbox_t mbox, mu_message_t msg,
-		 struct mu_auth_data *auth, const char *name,
+deliver_to_user (mu_mailbox_t mbox, mu_message_t msg,
+		 struct mu_auth_data *auth,
 		 char **errp)
 {
   int status;
@@ -159,9 +156,8 @@ deliver_to_user (mu_mailbox_t imbx, mu_mailbox_t mbox, mu_message_t msg,
   status = mu_mailbox_open (mbox, MU_STREAM_APPEND|MU_STREAM_CREAT);
   if (status != 0)
     {
-      maidag_error (_("Cannot open mailbox %s: %s"), 
+      maidag_error (_("cannot open mailbox %s: %s"), 
                     path, mu_strerror (status));
-      mu_mailbox_destroy (&mbox);
       return EX_TEMPFAIL;
     }
 
@@ -177,71 +173,69 @@ deliver_to_user (mu_mailbox_t imbx, mu_mailbox_t mbox, mu_message_t msg,
 
       if (status)
 	{
-	  maidag_error (_("Cannot lock mailbox `%s': %s"), path,
+	  maidag_error (_("cannot lock mailbox `%s': %s"), path,
 		        mu_strerror (status));
-	  mu_mailbox_destroy (&mbox);
 	  exit_code = EX_TEMPFAIL;
 	  return EX_TEMPFAIL;
 	}
     }
   
 #if defined(USE_MAILBOX_QUOTAS)
-  {
-    mu_off_t n;
-    mu_off_t msg_size;
-    mu_off_t mbsize;
-
-    if ((status = mu_mailbox_get_size (mbox, &mbsize)))
-      {
-	maidag_error (_("Cannot get size of mailbox %s: %s"),
-		      path, mu_strerror (status));
-	if (status == ENOSYS)
-	  mbsize = 0; /* Try to continue anyway */
-	else
-	  {
-	    mu_mailbox_destroy (&mbox);
+  if (auth)
+    {
+      mu_off_t n;
+      size_t msg_size;
+      mu_off_t mbsize;
+      
+      if ((status = mu_mailbox_get_size (mbox, &mbsize)))
+	{
+	  maidag_error (_("cannot get size of mailbox %s: %s"),
+			path, mu_strerror (status));
+	  if (status == ENOSYS)
+	    mbsize = 0; /* Try to continue anyway */
+	  else
 	    return EX_TEMPFAIL;
-	  }
-      }
+	}
     
-    switch (check_quota (auth, mbsize, &n))
-      {
-      case MQUOTA_EXCEEDED:
-	maidag_error (_("%s: mailbox quota exceeded for this recipient"), name);
-	if (errp)
-	  asprintf (errp, "%s: mailbox quota exceeded for this recipient",
-		    name);
-	exit_code = EX_QUOTA();
-	failed++;
-	break;
-	
-      case MQUOTA_UNLIMITED:
-	break;
-	
-      default:
-	if ((status = mu_mailbox_get_size (imbx, &msg_size)))
-	  {
-	    maidag_error (_("Cannot get message size (input message %s): %s"),
-			  path, mu_strerror (status));
-	    exit_code = EX_UNAVAILABLE;
-	    failed++;
-	  }
-	else if (msg_size > n)
-	  {
-	    maidag_error (_("%s: message would exceed maximum mailbox size for "
-			  "this recipient"),
-			  name);
-	    if (errp)
-	      asprintf (errp,
-			"%s: message would exceed maximum mailbox size "
-			"for this recipient",
-			name);
-	    exit_code = EX_QUOTA();
-	    failed++;
-	  }
-	break;
-      }
-  }
+      switch (check_quota (auth, mbsize, &n))
+	{
+	case MQUOTA_EXCEEDED:
+	  maidag_error (_("%s: mailbox quota exceeded for this recipient"),
+			auth->name);
+	  if (errp)
+	    asprintf (errp, "%s: mailbox quota exceeded for this recipient",
+		      auth->name);
+	  exit_code = EX_QUOTA();
+	  failed++;
+	  break;
+	  
+	case MQUOTA_UNLIMITED:
+	  break;
+	  
+	default:
+	  if ((status = mu_message_size (msg, &msg_size)))
+	    {
+	      maidag_error (_("cannot get message size (input message %s): %s"),
+			    path, mu_strerror (status));
+	      exit_code = EX_UNAVAILABLE;
+	      failed++;
+	    }
+	  else if (msg_size > n)
+	    {
+	      maidag_error (_("%s: message would exceed maximum mailbox size for "
+			      "this recipient"),
+			    auth->name);
+	      if (errp)
+		asprintf (errp,
+			  "%s: message would exceed maximum mailbox size "
+			  "for this recipient",
+			  auth->name);
+	      exit_code = EX_QUOTA();
+	      failed++;
+	    }
+	  break;
+	}
+    }
 #endif
   
   if (!failed && switch_user_id (auth, 1) == 0)
@@ -249,7 +243,7 @@ deliver_to_user (mu_mailbox_t imbx, mu_mailbox_t mbox, mu_message_t msg,
       status = mu_mailbox_append_message (mbox, msg);
       if (status)
 	{
-	  maidag_error (_("Error writing to mailbox %s: %s"),
+	  maidag_error (_("error writing to mailbox %s: %s"),
 		        path, mu_strerror (status));
 	  failed++;
 	}
@@ -258,7 +252,7 @@ deliver_to_user (mu_mailbox_t imbx, mu_mailbox_t mbox, mu_message_t msg,
 	  status = mu_mailbox_sync (mbox);
 	  if (status)
 	    {
-	      maidag_error (_("Error flushing mailbox %s: %s"),
+	      maidag_error (_("error flushing mailbox %s: %s"),
 			    path, mu_strerror (status));
 	      failed++;
 	    }
@@ -266,52 +260,108 @@ deliver_to_user (mu_mailbox_t imbx, mu_mailbox_t mbox, mu_message_t msg,
       switch_user_id (auth, 0);
     }
 
-  mu_auth_data_free (auth);
   mu_mailbox_close (mbox);
   mu_locker_unlock (lock);
-  mu_mailbox_destroy (&mbox);
   return failed ? exit_code : 0;
 }
 
-int
-deliver (mu_mailbox_t imbx, char *name, char **errp)
+static int
+is_remote_url (mu_url_t url)
 {
-  struct mu_auth_data *auth;
+  const char *scheme;
+  int rc = mu_url_sget_scheme (url, &scheme);
+  return rc == 0 && strncmp (scheme, "remote+", 7) == 0;
+}
+
+static int
+is_mailer_url (mu_url_t url)
+{
+  mu_record_t record = NULL;
+  int (*pfn) (mu_mailer_t) = NULL;
+  
+  return mu_registrar_lookup_url (url, MU_FOLDER_ATTRIBUTE_FILE,
+				  &record, NULL) == 0
+         && mu_record_get_mailer (record, &pfn) == 0
+         && pfn;
+}
+
+int
+deliver_url (mu_url_t url, mu_message_t msg, const char *name, char **errp)
+{
+  struct mu_auth_data *auth = NULL;
   mu_mailbox_t mbox;
-  mu_message_t msg;
   int status;
 
-  auth = mu_get_auth_by_name (name);
-  if (!auth)
+  if (name)
     {
-      maidag_error (_("%s: no such user"), name);
-      if (errp)
-	asprintf (errp, "%s: no such user", name);
-      exit_code = EX_UNAVAILABLE;
-      return EX_UNAVAILABLE;
+      auth = mu_get_auth_by_name (name);
+      if (!auth)
+	{
+	  maidag_error (_("%s: no such user"), name);
+	  if (errp)
+	    asprintf (errp, "%s: no such user", name);
+	  exit_code = EX_NOUSER;
+	  return EX_NOUSER;
+	}
+
+      if (current_uid)
+	auth->change_uid = 0;
+
+      if (switch_user_id (auth, 1))
+	return EX_TEMPFAIL;
+      status = script_apply (msg, auth);
+      if (switch_user_id (auth, 0))
+	return EX_TEMPFAIL;
+      if (status)
+	{
+	  exit_code = EX_OK;
+	  mu_auth_data_free (auth);
+	  return 0;
+	}
+ 
+      if (forward_file)
+	switch (maidag_forward (msg, auth, forward_file))
+	  {
+	  case maidag_forward_none:
+	  case maidag_forward_metoo:
+	    break;
+	    
+	  case maidag_forward_ok:
+	    mu_auth_data_free (auth);
+	    return 0;
+
+	  case maidag_forward_error:
+	    mu_auth_data_free (auth);
+	    return exit_code = EX_TEMPFAIL;
+	  }
     }
-  if (current_uid)
-    auth->change_uid = 0;
   
-  if (!sieve_test (auth, imbx))
+  if (!url)
     {
-      exit_code = EX_OK;
-      mu_auth_data_free (auth);
-      return 0;
-    }
+      status = mu_url_create (&url, auth->mailbox);
+      if (status)
+	{
+	  maidag_error (_("cannot create URL for %s: %s"),
+			auth->mailbox, mu_strerror (status));
+	  return exit_code = EX_UNAVAILABLE;
+	}
+      status = mu_url_parse (url);
+      if (status)
+	{
+	  maidag_error (_("error parsing URL %s: %s"),
+			auth->mailbox, mu_strerror (status));
+	  return exit_code = EX_UNAVAILABLE;
+	}
+    }      
 
-  if ((status = mu_mailbox_get_message (imbx, 1, &msg)) != 0)
+  status = mu_mailbox_create_from_url (&mbox, url);
+
+  if (status)
     {
-      maidag_error (_("Cannot get message from the temporary mailbox: %s"),
+      maidag_error (_("cannot open mailbox %s: %s"),
+		    mu_url_to_string (url),
 		    mu_strerror (status));
-      mu_auth_data_free (auth);
-      return EX_TEMPFAIL;
-    }
-
-  if ((status = mu_mailbox_create (&mbox, auth->mailbox)) != 0)
-    {
-      maidag_error (_("Cannot open mailbox %s: %s"),
-		    auth->mailbox, mu_strerror (status));
+      mu_url_destroy (&url);
       mu_auth_data_free (auth);
       return EX_TEMPFAIL;
     }
@@ -323,8 +373,67 @@ deliver (mu_mailbox_t imbx, char *name, char **errp)
      will be created */
   if (switch_user_id (auth, 1))
     return EX_TEMPFAIL;
-  status = deliver_to_user (imbx, mbox, msg, auth, name, errp);
+  status = deliver_to_user (mbox, msg, auth, errp);
   if (switch_user_id (auth, 0))
     return EX_TEMPFAIL;
+
+  mu_auth_data_free (auth);
+  mu_mailbox_destroy (&mbox);
+
   return status;
 }
+
+int
+deliver (mu_message_t msg, char *dest_id, char **errp)
+{
+  int status;
+  const char *name;
+  mu_url_t url = NULL;
+  
+  if (url_option)
+    {
+      status = mu_url_create (&url, dest_id);
+      if (status)
+	{
+	  maidag_error (_("%s: cannot create url: %s"), dest_id,
+			mu_strerror (status));
+	  return EX_NOUSER;
+	}
+      status = mu_url_parse (url);
+      if (status)
+	{
+	  maidag_error (_("%s: cannot parse url: %s"), dest_id,
+			mu_strerror (status));
+	  mu_url_destroy (&url);
+	  return EX_NOUSER;
+	}
+      status = mu_url_sget_user (url, &name);
+      if (status == MU_ERR_NOENT)
+	{
+	  if (!is_mailer_url (url) && !is_remote_url (url))
+	    {
+	      maidag_error (_("no user name"));
+	      if (errp)
+		asprintf (errp, "no such user");
+	      exit_code = EX_NOUSER;
+	      return EX_NOUSER;
+	    }
+	  else
+	    name = NULL;
+	}
+      else if (status)
+	{
+	  maidag_error (_("%s: cannot get user name from url: %s"),
+			dest_id, mu_strerror (status));
+	  mu_url_destroy (&url);
+	  return EX_NOUSER;
+	}
+    }
+  else
+    {
+      name = dest_id;
+      dest_id = NULL;
+    }
+  return deliver_url (url, msg, name, errp);
+}
+  
