@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2005, 2006, 
-   2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2005, 2006, 2007, 2009, 2010 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -72,7 +72,7 @@ parse_headers (FILE *fp, compose_env_t *env)
       switch (state)
 	{
 	case STATE_INIT:
-	  if (!buf[0] || isspace (buf[0]))
+	  if (!buf[0] || mu_isspace (buf[0]))
 	    continue;
 	  else
 	    state = STATE_READ;
@@ -81,7 +81,7 @@ parse_headers (FILE *fp, compose_env_t *env)
 	case STATE_READ:
 	  if (buf[0] == 0)
 	    state = STATE_BODY;
-	  else if (isspace (buf[0]))
+	  else if (mu_isspace (buf[0]))
 	    {
 	      /* A continuation line */
 	      if (name)
@@ -112,7 +112,7 @@ parse_headers (FILE *fp, compose_env_t *env)
 	      if (p)
 		{
 		  *p++ = 0;
-		  while (*p && isspace (*p))
+		  while (*p && mu_isspace (*p))
 		    p++;
 		  value = strdup (p);
 		  name = strdup (buf);
@@ -164,7 +164,7 @@ escape_check_args (int argc, char **argv)
   if (argc == 1)
     {
       char *escape = "~";
-      util_getenv (&escape, "escape", Mail_env_string, 0);
+      mailvar_get (&escape, "escape", mailvar_type_string, 0);
       util_error (_("%c%s requires an argument"), escape[0], argv[0]);
       return 1;
     }
@@ -234,11 +234,11 @@ escape_sign (int argc MU_ARG_UNUSED, char **argv, compose_env_t *env MU_ARG_UNUS
 {
   char *p;
 
-  if (util_getenv (&p, isupper (argv[0][0]) ? "Sign" : "sign",
-		   Mail_env_string, 1) == 0)
+  if (mailvar_get (&p, mu_isupper (argv[0][0]) ? "Sign" : "sign",
+		   mailvar_type_string, 1) == 0)
     {
       fputs ("-- \n", ofile);
-      if (isupper (argv[0][0]))
+      if (mu_isupper (argv[0][0]))
 	{
 	  char *name = util_fullpath (p);
 	  FILE *fp = fopen (name, "r");
@@ -292,9 +292,12 @@ escape_deadletter (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED,
   FILE *dead = fopen (getenv ("DEAD"), "r");
   int c;
 
-  while ((c = fgetc (dead)) != EOF)
-    fputc (c, ofile);
-  fclose (dead);
+  if (dead)
+    {
+      while ((c = fgetc (dead)) != EOF)
+	fputc (c, ofile);
+      fclose (dead);
+    }
   return 0;
 }
 
@@ -312,7 +315,7 @@ run_editor (char *ed, char *arg)
 static int
 escape_run_editor (char *ed, int argc, char **argv, compose_env_t *env)
 {
-  if (!util_getenv (NULL, "editheaders", Mail_env_boolean, 0))
+  if (!mailvar_get (NULL, "editheaders", mailvar_type_boolean, 0))
     {
       char *filename;
       int fd = mu_tempfile (NULL, &filename);
@@ -410,29 +413,9 @@ escape_headers (int argc, char **argv, compose_env_t *env)
 int
 escape_insert (int argc, char **argv, compose_env_t *send_env MU_ARG_UNUSED)
 {
-  struct mail_env_entry *env;
-
   if (escape_check_args (argc, argv))
     return 1;
-  env = util_find_env (argv[1], 0);
-  if (env)
-    switch (env->type)
-      {
-      case Mail_env_string:
-	fprintf (ofile, "%s", env->value.string);
-	break;
-
-      case Mail_env_number:
-	fprintf (ofile, "%d", env->value.number);
-	break;
-
-      case Mail_env_boolean:
-	fprintf (ofile, "%s", env->set ? "yes" : "no");
-	break;
-
-      default:
-	break;
-      }
+  mailvar_variable_format (ofile, mailvar_find_variable (argv[1], 0), NULL);
   return 0;
 }
 
@@ -450,9 +433,10 @@ quote0 (msgset_t *mspec, mu_message_t mesg, void *data)
   size_t n = 0;
   char *prefix = "\t";
   
-  fprintf (stdout, _("Interpolating: %d\n"), mspec->msg_part[0]);
+  fprintf (stdout, _("Interpolating: %lu\n"),
+	   (unsigned long) mspec->msg_part[0]);
 
-  util_getenv (&prefix, "indentprefix", Mail_env_string, 0);
+  mailvar_get (&prefix, "indentprefix", mailvar_type_string, 0);
 
   if (*(int*)data)
     {
@@ -506,7 +490,7 @@ quote0 (msgset_t *mspec, mu_message_t mesg, void *data)
 int
 escape_quote (int argc, char **argv, compose_env_t *env)
 {
-  int lower = islower (argv[0][0]);
+  int lower = mu_islower (argv[0][0]);
   util_foreach_msg (argc, argv, MSG_NODELETED|MSG_SILENT, quote0, &lower);
   escape_continue ();
   return 0;
