@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2005, 2007, 2010 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -118,10 +119,10 @@ mail_file (int argc, char **argv)
       mbox = newbox;
       mu_mailbox_messages_count (mbox, &total);
       set_cursor (1);
-      if (util_getenv (NULL, "header", Mail_env_boolean, 0) == 0)
+      if (mailvar_get (NULL, "header", mailvar_type_boolean, 0) == 0)
 	{
 	  util_do_command ("summary");
-	  util_do_command ("z.");
+	  util_do_command ("headers");
 	}
       return 0;
     }
