@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2007, 2010 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -27,13 +28,13 @@ int
 mail_help (int argc, char **argv)
 {
   if (argc < 2)
-    return util_help (mail_command_table, NULL);
+    return mail_command_help (NULL);
   else
     {
       int status = 0;
 
       while (--argc)
-	status |= util_help (mail_command_table, *++argv);
+	status |= mail_command_help (*++argv);
 
       return status;
     }
