@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007, 2009, 2010 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -34,7 +35,7 @@ sig_handler (int signo)
   switch (signo)
     {
     case SIGINT:
-      if (util_getenv (NULL, "quit", Mail_env_boolean, 0) == 0)
+      if (mailvar_get (NULL, "quit", mailvar_type_boolean, 0) == 0)
 	exit (0);
       _interrupted++;
       break;
@@ -68,22 +69,22 @@ ml_got_interrupt ()
 static int
 ml_getc (FILE *stream)
 {
-    unsigned char c;
+  unsigned char c;
 
-    while (1)
-      {
-	if (read (fileno (stream), &c, 1) == 1)
-	  return c;
-	if (errno == EINTR)
-	  {
-	    if (_interrupted)
-	      break;
-	    /* keep going if we handled the signal */
-	  }
-	else
-	  break;
-      }
-    return EOF;
+  while (1)
+    {
+      if (read (fileno (stream), &c, 1) == 1)
+	return c;
+      if (errno == EINTR)
+	{
+	  if (_interrupted)
+	    break;
+	  /* keep going if we handled the signal */
+	}
+      else
+	break;
+    }
+  return EOF;
 }
 
 void
@@ -174,7 +175,7 @@ ml_readline_internal ()
 }
 
 char *
-ml_readline (char *prompt)
+ml_readline (const char *prompt)
 {
   if (interactive)
     return readline (prompt);
@@ -182,7 +183,7 @@ ml_readline (char *prompt)
 }
 
 char *
-ml_readline_with_intr (char *prompt)
+ml_readline_with_intr (const char *prompt)
 {
   char *str = ml_readline (prompt);
   if (_interrupted)
@@ -236,7 +237,7 @@ ml_command_completion (char *cmd, int start, int end)
   char **ret;
   char *p;
 
-  for (p = rl_line_buffer; p < rl_line_buffer + start && isspace (*p); p++)
+  for (p = rl_line_buffer; p < rl_line_buffer + start && mu_isblank (*p); p++)
     ;
   
   if (mu_argcv_get_n (p, end, NULL, NULL, &argc, &argv))
@@ -304,7 +305,7 @@ ml_attempted_completion_over ()
 
 /* Completion functions */
 char **
-no_compl (int argc ARG_UNUSED, char **argv ARG_UNUSED, int ws ARG_UNUSED)
+no_compl (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED, int ws MU_ARG_UNUSED)
 {
   ml_attempted_completion_over ();
   return NULL;
@@ -319,7 +320,7 @@ msglist_compl (int argc, char **argv, int ws)
 }
 
 char **
-msglist_file_compl (int argc, char **argv, int ws ARG_UNUSED)
+msglist_file_compl (int argc, char **argv, int ws MU_ARG_UNUSED)
 {
   if (argc == 1)
     ml_attempted_completion_over ();
@@ -562,32 +563,6 @@ alias_compl (int argc, char **argv, int ws)
   return rl_completion_matches (ws ? "" : argv[argc-1], alias_generator);
 }
 
-static char *
-var_generator (const char *text, int state)
-{
-  static var_iterator_t itr;
-  const char *p;
-  
-  if (!state)
-    p = var_iterate_first (text, &itr);
-  else
-    p = var_iterate_next (itr);
-
-  if (!p)
-    {
-      var_iterate_end (&itr);
-      return NULL;
-    }
-  return strdup (p);
-}
-
-char **
-var_compl (int argc, char **argv, int ws)
-{
-  ml_attempted_completion_over ();
-  return rl_completion_matches (ws ? "" : argv[argc-1], var_generator);
-}
-
 static char *
 mkfilename (const char *dir, const char *file)
 {
@@ -988,7 +963,7 @@ readline (char *prompt)
 }
 
 void
-ml_set_completion_append_character (int c ARG_UNUSED)
+ml_set_completion_append_character (int c MU_ARG_UNUSED)
 {
 }
 
