@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2005, 2007, 2010 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -23,27 +24,27 @@
  */
 
 static int
-top0 (msgset_t *mspec, message_t msg, void *data)
+top0 (msgset_t *mspec, mu_message_t msg, void *data)
 {
-  stream_t stream;
+  mu_stream_t stream;
   char buf[512];
   size_t n;
   off_t off;
   int lines;
   
-  if (util_getenv (&lines, "toplines", Mail_env_number, 1)
+  if (mailvar_get (&lines, "toplines", mailvar_type_number, 1)
       || lines < 0)
     return 1;
 
-  message_get_stream (msg, &stream);
+  mu_message_get_stream (msg, &stream);
   for (n = 0, off = 0; lines > 0; lines--, off += n)
     {
-      int status = stream_readline (stream, buf, sizeof (buf), off, &n);
+      int status = mu_stream_readline (stream, buf, sizeof (buf), off, &n);
       if (status != 0 || n == 0)
 	break;
       fprintf (ofile, "%s", buf);
     }
-  cursor = mspec->msg_part[0];
+  set_cursor (mspec->msg_part[0]);
 
   util_mark_read (msg);
 
