@@ -1,10 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 
-   2004, 2005, 2006 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
+   2009, 2010 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -34,10 +34,6 @@
 #endif
 #include <mu_asprintf.h>
 
-mu_list_t environment = NULL;
-
-#define is_ascii_alpha(c) (isascii(c) && isalpha(c))
-
 /*
  * expands command into its command and arguments, then runs command
  * cmd is the command to parse and run
@@ -52,7 +48,6 @@ util_do_command (const char *c, ...)
   const struct mail_command_entry *entry = NULL;
   char *cmd = NULL;
   va_list ap;
-  static const char *delim = "=";
   
   va_start (ap, c);
   status = vasprintf (&cmd, c, ap);
@@ -82,7 +77,7 @@ util_do_command (const char *c, ...)
 	    return 0;
 	}
 
-      if (mu_argcv_get (cmd, delim, NULL, &argc, &argv) == 0 && argc > 0)
+      if (mu_argcv_get (cmd, NULL, NULL, &argc, &argv) == 0 && argc > 0)
 	{
 	  char *p;
 
@@ -93,7 +88,7 @@ util_do_command (const char *c, ...)
 	    {
 	      asprintf (&p, "print %s", argv[0]);
 	      mu_argcv_free (argc, argv);
-	      mu_argcv_get (p, delim, NULL, &argc, &argv);
+	      mu_argcv_get (p, NULL, NULL, &argc, &argv);
 	      free (p);
 	    }
 
@@ -112,7 +107,7 @@ util_do_command (const char *c, ...)
       char *p;
       
       for (p = argv[0] + strlen (argv[0]) - 1;
-	   p > argv[0] && !is_ascii_alpha (*p);
+	   p > argv[0] && !mu_isalpha (*p);
 	   p--)
 	;
 
@@ -272,13 +267,14 @@ util_help (void *table, size_t nmemb, size_t size, const char *word)
       FILE *out = stdout;
       char *p;
 
-      if (util_getenv (NULL, "crt", Mail_env_boolean, 0) == 0)
+      if (mailvar_get (NULL, "crt", mailvar_type_boolean, 0) == 0)
 	out = popen (getenv ("PAGER"), "w");
 
-  
       for (p = table, i = 0; i < nmemb; i++, p += size)
 	{
 	  struct mail_command *cp = (struct mail_command *)p;
+	  if (cp->synopsis == NULL)
+	    continue;
 	  fprintf (out, "%s\n", cp->synopsis);
 	}
       
@@ -291,7 +287,7 @@ util_help (void *table, size_t nmemb, size_t size, const char *word)
     {
       int status = 0;
       struct mail_command *cp = util_find_entry (table, nmemb, size, word);
-      if (cp)
+      if (cp && cp->synopsis)
 	fprintf (stdout, "%s\n", cp->synopsis);
       else
 	{
@@ -338,24 +334,6 @@ util_command_list (void *table, size_t nmemb, size_t size)
   return 0;
 }
 
-/*
- * removes whitespace from the beginning and end of a string
- */
-char *
-util_stripwhite (char *string)
-{
-  register char *s, *t;
-  for (s = string; isspace ((unsigned)*s); s++)
-    ;
-  if (*s == 0)
-    return s;
-  t = s + strlen (s) - 1;
-  while (t > s && isspace ((unsigned)*t))
-    t--;
-  *++t = '\0';
-  return s;
-}
-
 /*
  * Get the number of columns on the screen
  * First try an ioctl() call not all shells set the COLUMNS environ.
@@ -406,10 +384,10 @@ util_screen_lines ()
   int screen;
   size_t n;
 
-  if (util_getenv (&screen, "screen", Mail_env_number, 0) == 0)
+  if (mailvar_get (&screen, "screen", mailvar_type_number, 0) == 0)
     return screen;
   n = util_getlines();
-  util_do_command ("set screen=%d", n);
+  util_do_command ("set screen=%lu", (unsigned long) n);
   return n;
 }
 
@@ -419,10 +397,10 @@ util_screen_columns ()
   int cols;
   size_t n;
 
-  if (util_getenv (&cols, "columns", Mail_env_number, 0) == 0)
+  if (mailvar_get (&cols, "columns", mailvar_type_number, 0) == 0)
     return cols;
   n = util_getcols();
-  util_do_command ("set columns=%d", n);
+  util_do_command ("set columns=%lu", (unsigned long) n);
   return n;
 }
 
@@ -431,299 +409,18 @@ util_get_crt ()
 {
   int lines;
 
-  if (util_getenv (&lines, "crt", Mail_env_number, 0) == 0)
+  if (mailvar_get (&lines, "crt", mailvar_type_number, 0) == 0)
     return lines;
-  else if (util_getenv (NULL, "crt", Mail_env_boolean, 0) == 0)
+  else if (mailvar_get (NULL, "crt", mailvar_type_boolean, 0) == 0)
     return util_getlines ();
   return 0;
 }
 
-/* Functions for dealing with internal environment variables */
-
-/* Retrieve the value of a specified variable of given type.
-   The value is stored in the location pointed to by PTR variable.
-   VARIABLE and TYPE specify the variable name and type. If the
-   variable is not found and WARN is not null, the warning message
-   is issued.
-
-   Return value is 0 if the variable is found, 1 otherwise.
-   If PTR is not NULL, it must point to
-
-   int           if TYPE is Mail_env_number or Mail_env_boolean
-   const char *  if TYPE is Mail_env_string. 
-
-   Passing PTR=NULL may be used to check whether the variable is set
-   without retrieving its value. */
-   
-int
-util_getenv (void *ptr, const char *variable, mail_env_data_t type, int warn)
-{
-  struct mail_env_entry *env = util_find_env (variable, 0);
-
-  if (!mail_env_entry_is_set (env) || env->type != type)
-    {
-      if (warn)
-	util_error (_("No value set for \"%s\""), variable);
-      return 1;
-    }
-  if (ptr)
-    switch (type)
-      {
-      case Mail_env_string:
-	*(char**)ptr = env->value.string;
-	break;
-
-      case Mail_env_number:
-	*(int*)ptr = env->value.number;
-	break;
-
-      case Mail_env_boolean:
-	*(int*)ptr = env->value.bool;
-	break;
-
-      default:
-	break;
-      }
-	
-  return 0;
-}
-
-static int
-env_comp (const void *a, const void *b)
-{
-  const struct mail_env_entry *epa = a;
-  const struct mail_env_entry *epb = b;
-
-  return strcmp (epa->var, epb->var);
-}
-
-/* Find environment entry var. If not found and CREATE is not null, then
-   create the (unset and untyped) variable */
-struct mail_env_entry *
-util_find_env (const char *var, int create)
-{
-  struct mail_env_entry entry, *p;
-
-  if (strcmp (var, "ask") == 0)
-    entry.var = "asksub";
-  else
-    entry.var = var;
-  
-  if (environment == NULL)
-    {
-      mu_list_create (&environment);
-      mu_list_set_comparator (environment, env_comp);
-    }
-  
-  if (mu_list_locate (environment, &entry, (void**)&p))
-    {
-      if (!create)
-	return 0;
-	
-      p = xmalloc (sizeof *p);
-      p->var = xstrdup (entry.var);
-      mu_list_prepend (environment, p);
-      p->set = 0;
-      p->type = Mail_env_whatever;
-      p->value.number = 0;
-    }
-
-  return p;
-}
-
-struct var_iterator
-{
-  const char *prefix;
-  int prefixlen;
-  mu_iterator_t itr;
-};
-  
-const char *
-var_iterate_next (var_iterator_t itr)
-{
-  struct mail_env_entry *ep;
-  
-  while (!mu_iterator_is_done (itr->itr))
-    {
-      if (mu_iterator_current (itr->itr, (void **)&ep))
-	return NULL;
-      mu_iterator_next (itr->itr);
-  
-      if (strlen (ep->var) >= itr->prefixlen
-	  && strncmp (ep->var, itr->prefix, itr->prefixlen) == 0)
-	return ep->var;
-    }
-  return NULL;
-}
-
-const char *
-var_iterate_first (const char *prefix, var_iterator_t *pitr)
-{
-  if (environment)
-    {
-      var_iterator_t itr = xmalloc (sizeof *itr);
-      itr->prefix = prefix;
-      itr->prefixlen = strlen (prefix);
-      mu_list_get_iterator (environment, &itr->itr);
-      mu_iterator_first (itr->itr);
-      *pitr = itr;
-      return var_iterate_next (itr);
-    }
-  *pitr = NULL;
-  return NULL;
-}
-
-void
-var_iterate_end (var_iterator_t *itr)
-{
-  mu_iterator_destroy (&(*itr)->itr);
-  free (*itr);
-  *itr = NULL;
-}
-
-/* print the environment */
-static int
-envp_comp (const void *a, const void *b)
-{
-  struct mail_env_entry * const *epa = a;
-  struct mail_env_entry * const *epb = b;
-
-  return strcmp ((*epa)->var, (*epb)->var);
-}
-
-void
-util_printenv (int set)
-{
-  struct mail_env_entry **ep;
-  size_t i, count = 0;
-  
-  mu_list_count (environment, &count);
-  ep = xcalloc (count, sizeof *ep);
-  mu_list_to_array (environment, (void**) ep, count, NULL);
-  qsort (ep, count, sizeof *ep, envp_comp);
-  for (i = 0; i < count; i++)
-    {
-      switch (ep[i]->type)
-	{
-	case Mail_env_number:
-	  fprintf (ofile, "%s=%d", ep[i]->var, ep[i]->value.number);
-	  break;
-	  
-	case Mail_env_string:
-	  fprintf (ofile, "%s=\"%s\"", ep[i]->var, ep[i]->value.string);
-	  break;
-	  
-	case Mail_env_boolean:
-	  if (!ep[i]->value.bool)
-	    fprintf (ofile, "no");
-	  fprintf (ofile, "%s", ep[i]->var);
-	  break;
-	  
-	case Mail_env_whatever:
-	  fprintf (ofile, "%s %s", ep[i]->var, _("oops?"));
-	}
-      fprintf (ofile, "\n");
-    }
-  free (ep);
-}
-
-/* Initialize environment entry: clear set indicator and free any memory
-   associated with the data */
-void
-util_mail_env_free (struct mail_env_entry *ep)
-{
-  if (!mail_env_entry_is_set (ep))
-    return;
-  
-  switch (ep->type)
-    {
-    case Mail_env_string:
-      free (ep->value.string);
-      ep->value.string = NULL;
-      break;
-	      
-    default:
-      break;
-    }
-  ep->set = 0;
-}
-
-/* Set environement
-   The  util_setenv() function adds the variable name to the envi-
-   ronment with the value value, if  name  does  not  already
-   exist.   If  name  does exist in the environment, then its
-   value is changed to value if  overwrite  is  non-zero;  if
-   overwrite  is zero, then the value of name is not changed.
- 
-   A side effect of the code is if value is null the variable name
-   will be unset. */
-int
-util_setenv (const char *variable, void *value, mail_env_data_t type,
-	     int overwrite)
-{
-  struct mail_env_entry *ep =  util_find_env (variable, 1);
-
-  if (ep->set && !overwrite)
-    return 0;
-
-  util_mail_env_free (ep);
-  
-  ep->type = type;
-  if (value)
-    {
-      ep->set = 1;
-      switch (type)
-	{
-	case Mail_env_number:
-	  ep->value.number = *(int*)value;
-	  break;
-	  
-	case Mail_env_string:
-	  ep->value.string = strdup (value);
-	  break;
-	  
-	case Mail_env_boolean:
-	  ep->value.bool = *(int*)value;
-	  break;
-		  
-	default:
-	  abort();
-	}
-    }
-
-  /* Special handling for some variables */
-  if (strcmp (variable, "replyregex") == 0)
-    { 
-      int rc;
-      char *err;
-	      
-      if ((rc = mu_unre_set_regex (value, 0, &err)))
-	{
-	  fprintf (stderr, "%s", mu_strerror (rc));
-	  if (err)
-	    {
-	      fprintf (stderr, "%s", err);
-	      free (err);
-	    }
-	  fprintf (stderr, "\n");
-	}
-    }
-  else if (strcmp (variable, "decode-fallback") == 0)
-    {
-      if (mu_set_default_fallback (value))
-	mu_error (_("Incorrect value for decode-fallback"));
-    }
-  else if (strcmp (variable, "screen") == 0)
-    page_invalidate (1);
-  
-  return 0;
-}
-
 const char *
 util_reply_prefix ()
 {
   char *prefix = "Re: ";
-  util_getenv (&prefix, "replyprefix", Mail_env_string, 0);
+  mailvar_get (&prefix, "replyprefix", mailvar_type_string, 0);
   return prefix;
 }
 
@@ -771,7 +468,7 @@ util_get_homedir ()
 char *
 util_fullpath (const char *inpath)
 {
-  return mu_tilde_expansion(inpath, "/", NULL);
+  return mu_tilde_expansion (inpath, "/", NULL);
 }
 
 char *
@@ -781,7 +478,7 @@ util_folder_path (const char *name)
   char *tmp;
   char *p;
   
-  if (util_getenv (&folder, "folder", Mail_env_string, 1))
+  if (mailvar_get (&folder, "folder", mailvar_type_string, 1))
     return NULL;
       
   if (!name)
@@ -821,9 +518,9 @@ util_get_sender (int msgno, int strip)
   if (!addr)
     {
       mu_envelope_t env = NULL;
-      char buffer[512];
+      const char *buffer;
       mu_message_get_envelope (msg, &env);
-      if (mu_envelope_sender (env, buffer, sizeof (buffer), NULL)
+      if (mu_envelope_sget_sender (env, &buffer)
 	  || mu_address_create (&addr, buffer))
 	{
 	  util_error (_("Cannot determine sender name (msg %d)"), msgno);
@@ -868,7 +565,7 @@ util_slist_print (mu_list_t list, int nl)
 }
 
 int
-util_slist_lookup (mu_list_t list, char *str)
+util_slist_lookup (mu_list_t list, const char *str)
 {
   mu_iterator_t itr;
   char *name;
@@ -880,7 +577,7 @@ util_slist_lookup (mu_list_t list, char *str)
   for (mu_iterator_first (itr); !mu_iterator_is_done (itr); mu_iterator_next (itr))
     {
       mu_iterator_current (itr, (void **)&name);
-      if (strcasecmp (name, str) == 0)
+      if (mu_c_strcasecmp (name, str) == 0)
 	{
 	  rc = 1;
 	  break;
@@ -890,55 +587,39 @@ util_slist_lookup (mu_list_t list, char *str)
   return rc;
 }
 
-void
-util_slist_add (mu_list_t *list, char *value)
+static int
+util_slist_compare (const void *a, const void *b)
 {
-  char *p;
-
-  if (!*list && mu_list_create (list))
-    return;
-
-  if ((p = strdup(value)) == NULL)
-    {
-      util_error(_("Not enough memory"));
-      return;
-    }
-  mu_list_append (*list, p);
+  return mu_c_strcasecmp (a, b);
 }
 
-static int
-comp (const void *item, const void *data)
+void
+util_slist_add (mu_list_t *plist, char *value)
 {
-  return strcmp ((char*)item, (char*)data);
+  mu_list_t list;
+
+  if (!*plist)
+    {
+      if (mu_list_create (&list))
+	return;
+      mu_list_set_destroy_item (list, mu_list_free_item);
+      mu_list_set_comparator (list, util_slist_compare);
+      *plist = list;
+    }
+  else
+    list = *plist;
+  mu_list_append (list, xstrdup (value));
 }
 
 void
 util_slist_remove (mu_list_t *list, char *value)
 {
-  mu_list_comparator_t cp;
-  
-  if (!*list)
-    return;
-  cp = mu_list_set_comparator (*list, comp);
   mu_list_remove (*list, value);
-  mu_list_set_comparator (*list, cp);
 }
 
 void
 util_slist_destroy (mu_list_t *list)
 {
-  mu_iterator_t itr;
-  char *name;
-
-  if (!*list || mu_list_get_iterator (*list, &itr))
-    return;
-
-  for (mu_iterator_first (itr); !mu_iterator_is_done (itr); mu_iterator_next (itr))
-    {
-      mu_iterator_current (itr, (void **)&name);
-      free (name);
-    }
-  mu_iterator_destroy (&itr);
   mu_list_destroy (list);
 }
 
@@ -952,19 +633,20 @@ util_slist_to_string (mu_list_t list, const char *delim)
   if (!list || mu_list_get_iterator (list, &itr))
     return NULL;
 
-  for (mu_iterator_first (itr); !mu_iterator_is_done (itr); mu_iterator_next (itr))
+  for (mu_iterator_first (itr); !mu_iterator_is_done (itr);
+       mu_iterator_next (itr))
     {
       mu_iterator_current (itr, (void **)&name);
       if (str && delim)
-	util_strcat(&str, delim);
-      util_strcat(&str, name);
+	util_strcat (&str, delim);
+      util_strcat (&str, name);
     }
   mu_iterator_destroy (&itr);
   return str;
 }
 
 void
-util_strcat(char **dest, const char *str)
+util_strcat (char **dest, const char *str)
 {
   if (!*dest)
     *dest = strdup (str);
@@ -982,51 +664,6 @@ util_strcat(char **dest, const char *str)
     }
 }
 
-/* Upper case the entire string.  Assume it is NULL terminated.  */
-void
-util_strupper (char *s)
-{
-  if (s)
-    {
-      size_t i;
-      size_t len = strlen (s);
-      for (i = 0; i < len; i++)
-	s[i] = toupper ((unsigned int)(s[i]));
-    }
-}
-
-
-void
-util_escape_percent (char **str)
-{
-  int count;
-  char *p, *q;
-  char *newstr;
-
-  /* Count ocurrences of % in the string */
-  count = 0;
-  for (p = *str; *p; p++)
-    if (*p == '%')
-      count++;
-
-  if (!count)
-    return; /* nothing to do */
-
-  /* expand the string */
-  newstr = xmalloc (strlen (*str) + 1 + count);
-
-  /* and escape percent signs */
-  p = newstr;
-  q = *str;
-  while ((*p = *q++))
-    {
-      if (*p == '%')
-	*++p = '%';
-      p++;
-    }
-  *str = newstr;
-}
-
 char *
 util_outfolder_name (char *str)
 {
@@ -1047,7 +684,7 @@ util_outfolder_name (char *str)
       break;
 
     default:
-      if (util_getenv (&outfolder, "outfolder", Mail_env_string, 0) == 0)
+      if (mailvar_get (&outfolder, "outfolder", mailvar_type_string, 0) == 0)
 	{
 	  char *ns = NULL;
 	  asprintf (&ns, "%s/%s", outfolder, str);
@@ -1061,14 +698,14 @@ util_outfolder_name (char *str)
   return strdup (str);
 }
 
-/* Save an outgoing message. "savefile" allows to override the setting
+/* Save an outgoing message. The SAVEFILE argument overrides the setting
    of the "record" variable. */
 void
 util_save_outgoing (mu_message_t msg, char *savefile)
 {
   char *record;
   
-  if (util_getenv (&record, "record", Mail_env_string, 0) == 0)
+  if (mailvar_get (&record, "record", mailvar_type_string, 0) == 0)
     {
       int rc;
       mu_mailbox_t outbox;
@@ -1102,26 +739,13 @@ util_save_outgoing (mu_message_t msg, char *savefile)
     }
 }
 
-#ifdef HAVE_STDARG_H
 void
 util_error (const char *format, ...)
-#else
-void
-util_error (va_alist)
-     va_dcl
-#endif
 {
   va_list ap;
 
-#ifdef HAVE_STDARG_H
-  va_start(ap, format);
-#else
-  char *format;
-
-  va_start (ap);
-  format = va_arg (ap, char *);
-#endif
-
+  va_start (ap, format);
+  
   vfprintf (stderr, format, ap);
   fprintf (stderr, "\n");
 
@@ -1141,8 +765,8 @@ util_descend_subparts (mu_message_t mesg, msgset_t *msgset, mu_message_t *part)
       mu_header_t hdr = NULL;
 
       mu_message_get_header (mesg, &hdr);
-      util_get_content_type (hdr, &type);
-      if (strncasecmp (type, "message/rfc822", strlen (type)) == 0)
+      util_get_content_type (hdr, &type, NULL);
+      if (mu_c_strncasecmp (type, "message/rfc822", strlen (type)) == 0)
 	{
 	  if (mu_message_unencapsulate (mesg, &submsg, NULL))
 	    {
@@ -1155,15 +779,15 @@ util_descend_subparts (mu_message_t mesg, msgset_t *msgset, mu_message_t *part)
       mu_message_get_num_parts (mesg, &nparts);
       if (nparts < msgset->msg_part[i])
 	{
-	  util_error (_("No such (sub)part in the message: %d"),
-		      msgset->msg_part[i]);
+	  util_error (_("No such (sub)part in the message: %lu"),
+		      (unsigned long) msgset->msg_part[i]);
 	  return 1;
 	}
 
       if (mu_message_get_part (mesg, msgset->msg_part[i], &submsg))
 	{
-	  util_error (_("Cannot get (sub)part from the message: %d"),
-		      msgset->msg_part[i]);
+	  util_error (_("Cannot get (sub)part from the message: %lu"),
+		      (unsigned long) msgset->msg_part[i]);
 	  return 1;
 	}
 
@@ -1187,12 +811,12 @@ util_msgset_iterate (msgset_t *msgset,
 	return;
 
       if (util_descend_subparts (mesg, msgset, &mesg) == 0)
-	  (*fun)(mesg, msgset, closure);
+	(*fun) (mesg, msgset, closure);
     }
 }
 
 int
-util_get_content_type (mu_header_t hdr, char **value)
+util_get_content_type (mu_header_t hdr, char **value, char **args)
 {
   char *type = NULL;
   util_get_hdr_value (hdr, MU_HEADER_CONTENT_TYPE, &type);
@@ -1202,6 +826,17 @@ util_get_content_type (mu_header_t hdr, char **value)
 	free (type);
       type = strdup ("text/plain"); /* Default.  */
     }
+  else
+    {
+      char *p;
+      p = strchr (type, ';');
+      if (p)
+	{
+	  *p++ = 0;
+	  if (args)
+	    *args = p;
+	}
+    }
   *value = type;
   return 0;
 }
@@ -1209,16 +844,9 @@ util_get_content_type (mu_header_t hdr, char **value)
 int
 util_get_hdr_value (mu_header_t hdr, const char *name, char **value)
 {
-  int status = mu_header_aget_value (hdr, name, value);
+  int status = mu_header_aget_value_unfold (hdr, name, value);
   if (status == 0)
-    {
-      /* Remove the newlines.  */
-      char *nl;
-      while ((nl = strchr (*value, '\n')) != NULL)
-	{
-	  *nl = ' ';
-	}
-    }
+    mu_rtrim_class (*value, MU_CTYPE_SPACE);
   return status;
 }
 
@@ -1271,7 +899,7 @@ is_address_field (const char *name)
   char **p;
   
   for (p = address_fields; *p; p++)
-    if (strcasecmp (*p, name) == 0)
+    if (mu_c_strcasecmp (*p, name) == 0)
       return 1;
   return 0;
 }
@@ -1319,10 +947,10 @@ util_header_expand (mu_header_t *phdr)
 	    {
 	      mu_address_t new_addr;
 	      
-	      while (*p && isspace (*p))
+	      while (*p && mu_isspace (*p))
 		p++;
 	      /* If inplacealiases was set, the value was already expanded */
-	      if (util_getenv (NULL, "inplacealiases", Mail_env_boolean, 0))
+	      if (mailvar_get (NULL, "inplacealiases", mailvar_type_boolean, 0))
 		exp = alias_expand (p);
 	      rc = mu_address_create (&new_addr, exp ? exp : p);
 	      if (rc)
@@ -1396,7 +1024,7 @@ util_get_message (mu_mailbox_t mbox, size_t msgno, mu_message_t *msg)
 int
 util_error_range (size_t msgno)
 {
-  util_error (_("%d: invalid message number"), msgno);
+  util_error (_("%lu: invalid message number"), (unsigned long) msgno);
   return 1;
 }
 
@@ -1425,7 +1053,7 @@ util_cache_command (mu_list_t *list, const char *fmt, ...)
 static int
 _run_and_free (void *item, void *data)
 {
-  util_do_command ((char *) item);
+  util_do_command ("%s", (char *) item);
   free (item);
   return 0;
 }
@@ -1441,14 +1069,14 @@ void
 util_rfc2047_decode (char **value)
 {
   char locale[32];
-  char *charset = NULL;
+  const char *charset = NULL;
   char *tmp;
   int rc;
 
-  if (!*value || util_getenv (&charset, "charset", Mail_env_string, 0))
+  if (!*value || mailvar_get (&charset, "charset", mailvar_type_string, 0))
     return;
 
-  if (strcasecmp (charset, "auto") == 0)
+  if (mu_c_strcasecmp (charset, "auto") == 0)
     {
       memset (locale, 0, sizeof (locale));
 
@@ -1481,7 +1109,7 @@ util_rfc2047_decode (char **value)
   rc = mu_rfc2047_decode (charset, *value, &tmp);
   if (rc)
     {
-      if (util_getenv (NULL, "verbose", Mail_env_boolean, 0) == 0)
+      if (mailvar_get (NULL, "verbose", mailvar_type_boolean, 0) == 0)
 	mu_error (_("Cannot decode line `%s': %s"), *value, mu_strerror (rc));
     }
   else
@@ -1491,3 +1119,18 @@ util_rfc2047_decode (char **value)
     }
 }
 
+const char *
+util_url_to_string (mu_url_t url)
+{
+  const char *scheme;
+  if (mu_url_sget_scheme (url, &scheme) == 0)
+    {
+      if (strcmp (scheme, "file") == 0 || strcmp (scheme, "mbox") == 0)
+	{
+	  const char *path;
+	  if (mu_url_sget_path (url, &path) == 0)
+	    return path;
+	}
+    }
+  return mu_url_to_string (url);
+}
