@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 
-   2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
+   2008, 2009, 2010 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -36,7 +36,6 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
-#include <ctype.h>
 #include <dirent.h>
 
 #ifdef WITH_PTHREAD
@@ -53,6 +52,8 @@
 # include <strings.h>
 #endif
 
+#include <mailutils/cctype.h>
+#include <mailutils/cstr.h>
 #include <mailutils/attribute.h>
 #include <mailutils/body.h>
 #include <mailutils/debug.h>
@@ -600,7 +601,7 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm,
 		   int expunge)
 {
   mu_stream_t stream = NULL;
-  char *name = NULL, *buf = NULL, *msg_name;
+  char *name = NULL, *buf = NULL, *msg_name, *old_name;
   size_t n, off = 0;
   size_t bsize;
   size_t nlines, nbytes;
@@ -613,14 +614,33 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm,
   mu_body_t body;
   const char *sbuf;
   mu_envelope_t env = NULL;
-  
+  char statbuf[MU_STATUS_BUF_SIZE];
+
   status = mu_message_size (msg, &bsize);
   if (status)
     return status;
 
+  status = amd->new_msg_file_name (mhm, mhm->attr_flags, expunge, &msg_name);
+  if (status)
+    return status;
+  if (!msg_name)
+    {
+      /* Unlink the original file */
+      char *old_name;
+      status = amd->cur_msg_file_name (mhm, &old_name);
+      free (msg_name);
+      if (status == 0 && unlink (old_name))
+	status = errno;
+      free (old_name);
+      return status;
+    }      
+    
   fp = _amd_tempfile (mhm->amd, &name);
   if (!fp)
-    return errno;
+    {
+      free (msg_name);
+      return errno;
+    }
 
   /* Try to allocate large buffer */
   for (; bsize > 1; bsize /= 2)
@@ -628,7 +648,12 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm,
       break;
 
   if (!bsize)
-    return ENOMEM;
+    {
+      unlink (name);
+      free (name);
+      free (msg_name);
+      return ENOMEM;
+    }
 
   /* Copy flags */
   mu_message_get_header (msg, &hdr);
@@ -638,14 +663,16 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm,
   while ((status = mu_stream_readline (stream, buf, bsize, off, &n)) == 0
 	 && n != 0)
     {
-      if (_amd_delim(buf))
+      if (_amd_delim (buf))
 	break;
 
-      if (!(strncasecmp (buf, "status:", 7) == 0
-	    || strncasecmp (buf, "x-imapbase:", 11) == 0
-	    || strncasecmp (buf, "x-uid:", 6) == 0
-	    || strncasecmp (buf, MU_HEADER_ENV_DATE ":", sizeof (MU_HEADER_ENV_DATE)) == 0
-	    || strncasecmp (buf, MU_HEADER_ENV_SENDER ":", sizeof (MU_HEADER_ENV_SENDER)) == 0))
+      if (!(mu_c_strncasecmp (buf, "status:", 7) == 0
+	    || mu_c_strncasecmp (buf, "x-imapbase:", 11) == 0
+	    || mu_c_strncasecmp (buf, "x-uid:", 6) == 0
+	    || mu_c_strncasecmp (buf, 
+                MU_HEADER_ENV_DATE ":", sizeof (MU_HEADER_ENV_DATE)) == 0
+	    || mu_c_strncasecmp (buf, 
+                MU_HEADER_ENV_SENDER ":", sizeof (MU_HEADER_ENV_SENDER)) == 0))
 	{
 	  nlines++;
 	  nbytes += fprintf (fp, "%s", buf);
@@ -669,7 +696,7 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm,
   if (mu_envelope_sget_date (env, &sbuf) == 0)
     {
       /* NOTE: buffer might be terminated with \n */
-      while (*sbuf && isspace (*sbuf))
+      while (*sbuf && mu_isspace (*sbuf))
 	sbuf++;
       nbytes += fprintf (fp, "%s: %s", MU_HEADER_ENV_DATE, sbuf);
 
@@ -687,10 +714,10 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm,
   
   /* Add status */
   mu_message_get_attribute (msg, &attr);
-  mu_attribute_to_string (attr, buf, bsize, &n);
+  mu_attribute_to_string (attr, statbuf, sizeof (statbuf), &n);
   if (n)
     {
-      nbytes += fprintf (fp, "%s", buf);
+      nbytes += fprintf (fp, "Status: %s\n", statbuf);
       nlines++;
     }
   nbytes += fprintf (fp, "\n");
@@ -724,42 +751,37 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm,
   free (buf);
   fclose (fp);
 
-  status = amd->new_msg_file_name (mhm, mhm->attr_flags, &msg_name);
+  status = amd->cur_msg_file_name (mhm, &old_name);
   if (status == 0)
     {
-      char *old_name;
-      status = amd->cur_msg_file_name (mhm, &old_name);
-      if (status == 0)
+      if (rename (name, msg_name))
+	status = errno;
+      else
 	{
-	  if (rename (name, msg_name))
-	    status = errno;
-	  else
-	    {
-	      mode_t perms;
+	  mode_t perms;
 	  
-	      perms = mu_stream_flags_to_mode (amd->mailbox->flags, 0);
-	      if (perms != 0)
-		{
-		  /* It is documented that the mailbox permissions are
-		     affected by the current umask, so take it into account
-		     here.
-		     FIXME: I'm still not sure we should honor umask, though.
-		     --gray
-		  */
-		  mode_t mask = umask (0);
-		  chmod (msg_name, (0600 | perms) & ~mask);
-		  umask (mask);
-		}
-	      if (strcmp (old_name, msg_name))
-		/* Unlink original message */
-		unlink (old_name);
+	  perms = mu_stream_flags_to_mode (amd->mailbox->flags, 0);
+	  if (perms != 0)
+	    {
+	      /* It is documented that the mailbox permissions are
+		 affected by the current umask, so take it into account
+		 here.
+		 FIXME: I'm still not sure we should honor umask, though.
+		 --gray
+	      */
+	      mode_t mask = umask (0);
+	      chmod (msg_name, (0600 | perms) & ~mask);
+	      umask (mask);
 	    }
-	  free (old_name);
-
-	  mhm->orig_flags = mhm->attr_flags;
+	  if (strcmp (old_name, msg_name))
+	    /* Unlink original message */
+	    unlink (old_name);
 	}
-      free (msg_name);
+      free (old_name);
+      
+      mhm->orig_flags = mhm->attr_flags;
     }
+  free (msg_name);
   free (name);
 
   return status;
@@ -1065,33 +1087,38 @@ amd_expunge (mu_mailbox_t mailbox)
       
       if (mhm->attr_flags & MU_ATTRIBUTE_DELETED)
 	{
-	  if (!(mhm->orig_flags & MU_ATTRIBUTE_DELETED))
-	    {
-	      int rc;
-	      char *old_name;
-	      char *new_name;
+	  int rc;
+	  char *old_name;
+	  char *new_name;
 
-	      rc = amd->cur_msg_file_name (mhm, &old_name);
-	      if (rc)
-		return rc;
-	      rc = amd->new_msg_file_name (mhm, mhm->attr_flags, &new_name);
-	      if (rc)
-		{
-		  free (old_name);
-		  return rc;
-		}
-	      
-	      if (new_name)
-		{
-		  /* Rename original message */
-		  rename (old_name, new_name);
-		  free (new_name);
-		}
-	      else
-		/* Unlink original file */
-		unlink (old_name);
+	  rc = amd->cur_msg_file_name (mhm, &old_name);
+	  if (rc)
+	    return rc;
+	  rc = amd->new_msg_file_name (mhm, mhm->attr_flags, 1,
+				       &new_name);
+	  if (rc)
+	    {
 	      free (old_name);
+	      return rc;
 	    }
+
+	  if (new_name)
+	    {
+	      /* FIXME: It may be a good idea to have a capability flag
+		 in struct _amd_data indicating that no actual removal
+		 is needed (e.g. for traditional MH). It will allow to
+		 bypass lots of no-op code here. */
+	      if (strcmp (old_name, new_name))
+		/* Rename original message */
+		rename (old_name, new_name);
+	    }
+	  else
+	    /* Unlink original file */
+	    unlink (old_name);
+	  
+	  free (old_name);
+	  free (new_name);
+
 	  _amd_message_delete (amd, mhm);
 	  updated = 1;
 	  /* Do not increase i! */
@@ -1314,13 +1341,13 @@ amd_scan_message (struct _amd_message *mhm)
 	    hlines++;
 
 	  /* Process particular attributes */
-	  if (strncasecmp (buf, "status:", 7) == 0)
+	  if (mu_c_strncasecmp (buf, "status:", 7) == 0)
 	    {
 	      int deleted = mhm->attr_flags & MU_ATTRIBUTE_DELETED;
 	      mu_string_to_flags (buf, &mhm->attr_flags);
 	      mhm->attr_flags |= deleted;
 	    }
-	  else if (strncasecmp (buf, "x-imapbase:", 11) == 0)
+	  else if (mu_c_strncasecmp (buf, "x-imapbase:", 11) == 0)
 	    {
 	      char *p;
 	      mhm->amd->uidvalidity = strtoul (buf + 11, &p, 10);
