@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2004, 2005, 2007, 2008 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2004, 2005, 2007, 2008, 2010 Free
+   Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -29,6 +29,14 @@
 #include <iterator0.h>
 #include <mailutils/errno.h>
 
+#define DESTROY_ITEM(list, elt)			\
+  do						\
+    {						\
+       if ((list)->destroy_item)		\
+	 (list)->destroy_item ((elt)->item);	\
+    }						\
+  while (0)
+
 int
 mu_list_create (mu_list_t *plist)
 {
@@ -40,14 +48,14 @@ mu_list_create (mu_list_t *plist)
   list = calloc (sizeof (*list), 1);
   if (list == NULL)
     return ENOMEM;
-  status = mu_monitor_create (&(list->monitor), 0,  list);
+  status = mu_monitor_create (&list->monitor, 0,  list);
   if (status != 0)
     {
       free (list);
       return status;
     }
-  list->head.next = &(list->head);
-  list->head.prev = &(list->head);
+  list->head.next = &list->head;
+  list->head.prev = &list->head;
   *plist = list;
   return 0;
 }
@@ -62,16 +70,15 @@ mu_list_destroy (mu_list_t *plist)
       struct list_data *previous;
 
       mu_monitor_wrlock (list->monitor);
-      for (current = list->head.next; current != &(list->head);)
+      for (current = list->head.next; current != &list->head;)
 	{
 	  previous = current;
 	  current = current->next;
-	  if (list->destroy_item)
-	    list->destroy_item (previous->item);
+	  DESTROY_ITEM (list, previous);
 	  free (previous);
 	}
       mu_monitor_unlock (list->monitor);
-      mu_monitor_destroy (&(list->monitor), list);
+      mu_monitor_destroy (&list->monitor, list);
       free (list);
       *plist = NULL;
     }
@@ -91,7 +98,7 @@ mu_list_append (mu_list_t list, void *item)
     return ENOMEM;
   ldata->item = item;
   mu_monitor_wrlock (list->monitor);
-  ldata->next = &(list->head);
+  ldata->next = &list->head;
   ldata->prev = list->head.prev;
   last->next = ldata;
   list->head.prev = ldata;
@@ -114,7 +121,7 @@ mu_list_prepend (mu_list_t list, void *item)
     return ENOMEM;
   ldata->item = item;
   mu_monitor_wrlock (list->monitor);
-  ldata->prev = &(list->head);
+  ldata->prev = &list->head;
   ldata->next = list->head.next;
   first->prev = ldata;
   list->head.next = ldata;
@@ -164,8 +171,8 @@ mu_list_get_comparator (mu_list_t list, mu_list_comparator_t *comp)
   return 0;
 }
 
-static int
-def_comp (const void *item, const void *value)
+int
+_mu_list_ptr_comparator (const void *item, const void *value)
 {
   return item != value;
 }
@@ -179,10 +186,10 @@ mu_list_locate (mu_list_t list, void *item, void **ret_item)
 
   if (list == NULL)
     return EINVAL;
-  comp = list->comp ? list->comp : def_comp;
+  comp = list->comp ? list->comp : _mu_list_ptr_comparator;
   mu_monitor_wrlock (list->monitor);
-  for (previous = &(list->head), current = list->head.next;
-       current != &(list->head); previous = current, current = current->next)
+  for (previous = &list->head, current = list->head.next;
+       current != &list->head; previous = current, current = current->next)
     {
       if (comp (current->item, item) == 0)
 	{
@@ -197,40 +204,23 @@ mu_list_locate (mu_list_t list, void *item, void **ret_item)
 }
 
 static int
-_insert_item(mu_list_t list, struct list_data *current, void *new_item,
-	     int insert_before)
+_insert_item (mu_list_t list, struct list_data *current, void *new_item,
+	      int insert_before)
 {
+  int status;
   struct list_data *ldata = calloc (sizeof (*ldata), 1);
   if (ldata == NULL)
-    return ENOMEM;
-
-  ldata->item = new_item;
-  if (insert_before)
-    {
-      ldata->prev = current->prev;
-      ldata->next = current;
-      if (current->prev != &list->head)
-	current->prev->next = ldata;
-      else
-	list->head.next = ldata;
-
-      current->prev = ldata;
-    }
+    status = ENOMEM;
   else
     {
-      ldata->next = current->next;
-      ldata->prev = current;
-      
-      if (current->next != &list->head)
-	current->next->prev = ldata;
-      else
-	list->head.prev = ldata;
-
-      current->next = ldata;
+      ldata->item = new_item;
+      _mu_list_insert_sublist (list, current,
+			       ldata, ldata,
+			       1,
+			       insert_before);
+      status = 0;
     }
-  
-  list->count++;
-  return 0;
+  return status;
 }
 
 int
@@ -242,11 +232,11 @@ mu_list_insert (mu_list_t list, void *item, void *new_item, int insert_before)
 
   if (list == NULL)
     return EINVAL;
-  comp = list->comp ? list->comp : def_comp;
+  comp = list->comp ? list->comp : _mu_list_ptr_comparator;
 
   mu_monitor_wrlock (list->monitor);
   for (current = list->head.next;
-       current != &(list->head);
+       current != &list->head;
        current = current->next)
     {
       if (comp (current->item, item) == 0)
@@ -262,22 +252,25 @@ mu_list_insert (mu_list_t list, void *item, void *new_item, int insert_before)
 int
 mu_list_remove (mu_list_t list, void *item)
 {
-  struct list_data *current, *previous;
+  struct list_data *current;
   mu_list_comparator_t comp;
   int status = MU_ERR_NOENT;
 
   if (list == NULL)
     return EINVAL;
-  comp = list->comp ? list->comp : def_comp;
+  comp = list->comp ? list->comp : _mu_list_ptr_comparator;
   mu_monitor_wrlock (list->monitor);
-  for (previous = &(list->head), current = list->head.next;
-       current != &(list->head); previous = current, current = current->next)
+  for (current = list->head.next;
+       current != &list->head; current = current->next)
     {
       if (comp (current->item, item) == 0)
 	{
+	  struct list_data *previous = current->prev;
+	  
 	  mu_iterator_advance (list->itr, current);
 	  previous->next = current->next;
 	  current->next->prev = previous;
+	  DESTROY_ITEM (list, current);
 	  free (current);
 	  list->count--;
 	  status = 0;
@@ -288,6 +281,15 @@ mu_list_remove (mu_list_t list, void *item)
   return status;
 }
 
+int
+mu_list_remove_nd (mu_list_t list, void *item)
+{
+  mu_list_destroy_item_t dptr = mu_list_set_destroy_item (list, NULL);
+  int rc = mu_list_remove (list, item);
+  mu_list_set_destroy_item (list, dptr);
+  return rc;
+}
+
 int
 mu_list_replace (mu_list_t list, void *old_item, void *new_item)
 {
@@ -297,13 +299,14 @@ mu_list_replace (mu_list_t list, void *old_item, void *new_item)
 
   if (list == NULL)
     return EINVAL;
-  comp = list->comp ? list->comp : def_comp;
+  comp = list->comp ? list->comp : _mu_list_ptr_comparator;
   mu_monitor_wrlock (list->monitor);
-  for (previous = &(list->head), current = list->head.next;
-       current != &(list->head); previous = current, current = current->next)
+  for (previous = &list->head, current = list->head.next;
+       current != &list->head; previous = current, current = current->next)
     {
       if (comp (current->item, old_item) == 0)
 	{
+	  DESTROY_ITEM (list, current);
 	  current->item = new_item;
 	  status = 0;
 	  break;
@@ -313,6 +316,15 @@ mu_list_replace (mu_list_t list, void *old_item, void *new_item)
   return status;
 }
 
+int
+mu_list_replace_nd (mu_list_t list, void *item, void *new_item)
+{
+  mu_list_destroy_item_t dptr = mu_list_set_destroy_item (list, NULL);
+  int rc = mu_list_replace (list, item, new_item);
+  mu_list_set_destroy_item (list, dptr);
+  return rc;
+}
+
 int
 mu_list_get (mu_list_t list, size_t indx, void **pitem)
 {
@@ -325,7 +337,7 @@ mu_list_get (mu_list_t list, size_t indx, void **pitem)
   if (pitem == NULL)
     return MU_ERR_OUT_PTR_NULL;
   mu_monitor_rdlock (list->monitor);
-  for (current = list->head.next, count = 0; current != &(list->head);
+  for (current = list->head.next, count = 0; current != &list->head;
        current = current->next, count++)
     {
       if (count == indx)
@@ -340,17 +352,18 @@ mu_list_get (mu_list_t list, size_t indx, void **pitem)
 }
 
 int
-mu_list_do (mu_list_t list, mu_list_action_t * action, void *cbdata)
+mu_list_do (mu_list_t list, mu_list_action_t *action, void *cbdata)
 {
   mu_iterator_t itr;
   int status = 0;
 
   if (list == NULL || action == NULL)
     return EINVAL;
-  status = mu_list_get_iterator(list, &itr);
+  status = mu_list_get_iterator (list, &itr);
   if (status)
     return status;
-  for (mu_iterator_first (itr); !mu_iterator_is_done (itr); mu_iterator_next (itr))
+  for (mu_iterator_first (itr); !mu_iterator_is_done (itr);
+       mu_iterator_next (itr))
     {
       void *item;
       mu_iterator_current (itr, &item);
@@ -361,13 +374,12 @@ mu_list_do (mu_list_t list, mu_list_action_t * action, void *cbdata)
   return status;
 }
 
-int
+mu_list_destroy_item_t
 mu_list_set_destroy_item (mu_list_t list, void (*destroy_item)(void *item))
 {
-  if (list == NULL)
-    return EINVAL;
+  mu_list_destroy_item_t ret = list->destroy_item;
   list->destroy_item = destroy_item;
-  return 0;
+  return ret;
 }
 
 int
@@ -386,7 +398,7 @@ mu_list_to_array (mu_list_t list, void **array, size_t count, size_t *pcount)
       struct list_data *current;
 
       for (i = 0, current = list->head.next;
-	   i < total && current != &(list->head); current = current->next)
+	   i < total && current != &list->head; current = current->next)
 	array[i++] = current->item;
     }
   if (pcount)
@@ -394,15 +406,15 @@ mu_list_to_array (mu_list_t list, void **array, size_t count, size_t *pcount)
   return 0;
 }
 
-/* Computes an intersection of the two lists. The resulting list,
-   stored into PDEST, contains elements from the list A that are
-   also encountered in the list B, using the comparison function of
-   the latter.
+/* Computes an intersection of two lists and returns it in PDEST.
+   The resulting list contains elements from A that are
+   also encountered in B (as per comparison function of
+   the latter).
 
-   If DUP_ITEM is not NULL, it is used to create copies of the
+   If DUP_ITEM is not NULL, it is used to create copies of
    items to be stored in PDEST.  In this case, the destroy_item
    function of B is also attached to PDEST.  Otherwise, if
-   DUP_ITEM is NULL, the pointers to the elements are stored and
+   DUP_ITEM is NULL, pointers to elements are stored and
    no destroy_item function is assigned. */
 int
 mu_list_intersect_dup (mu_list_t *pdest, mu_list_t a, mu_list_t b,
@@ -467,13 +479,17 @@ struct list_iterator
 {
   mu_list_t list;
   struct list_data *cur;
+  int backwards; /* true if iterating backwards */
 };
 
 static int
 first (void *owner)
 {
   struct list_iterator *itr = owner;
-  itr->cur = itr->list->head.next;
+  if (itr->backwards)
+    itr->cur = itr->list->head.prev;
+  else
+    itr->cur = itr->list->head.next;
   return 0;
 }
 
@@ -481,7 +497,10 @@ static int
 next (void *owner)
 {
   struct list_iterator *itr = owner;
-  itr->cur = itr->cur->next;
+  if (itr->backwards)
+    itr->cur = itr->cur->prev;
+  else
+    itr->cur = itr->cur->next;
   return 0;
 }
 
@@ -528,6 +547,105 @@ list_data_dup (void **ptr, void *owner)
   return 0;
 }
 
+static int
+list_itrctl (void *owner, enum mu_itrctl_req req, void *arg)
+{
+  struct list_iterator *itr = owner;
+  mu_list_t list = itr->list;
+  struct list_data *ptr;
+  
+  if (itr->cur == NULL)
+    return MU_ERR_NOENT;
+  switch (req)
+    {
+    case mu_itrctl_tell:
+      /* Return current position in the object */
+      {
+	size_t count;
+
+	for (count = 0, ptr = list->head.next; ptr != &list->head;
+	     ptr = ptr->next, count++)
+	  {
+	    if (ptr == itr->cur)
+	      {
+		*(size_t*)arg = count;
+		return 0;
+	      }
+	  }
+	return MU_ERR_NOENT;
+      }
+	
+    case mu_itrctl_delete:
+    case mu_itrctl_delete_nd:
+      /* Delete current element */
+      {
+	struct list_data *prev;
+	
+	ptr = itr->cur;
+	prev = ptr->prev;
+	
+	mu_iterator_advance (list->itr, ptr);
+	prev->next = ptr->next;
+	ptr->next->prev = prev;
+	if (req == mu_itrctl_delete)
+	  DESTROY_ITEM (list, ptr);
+	free (ptr);
+	list->count--;
+      }
+      break;
+      
+    case mu_itrctl_replace:
+    case mu_itrctl_replace_nd:
+      /* Replace current element */
+      if (!arg)
+	return EINVAL;
+      if (req == mu_itrctl_replace)
+	  DESTROY_ITEM (list, ptr);
+      ptr = itr->cur;
+      ptr->item = arg;
+      break;
+      
+    case mu_itrctl_insert:
+      /* Insert new element in the current position */
+      if (!arg)
+	return EINVAL;
+      return _insert_item (list, itr->cur, arg, 0);
+
+    case mu_itrctl_insert_list:
+      /* Insert a list of elements */
+      if (!arg)
+	return EINVAL;
+      else
+	{
+	  mu_list_t new_list = arg;
+	  _mu_list_insert_sublist (list, itr->cur,
+				   new_list->head.next, new_list->head.prev,
+				   new_list->count,
+				   0);
+	  _mu_list_clear (new_list);
+	}
+      break;
+
+    case mu_itrctl_qry_direction:
+      if (!arg)
+	return EINVAL;
+      else
+	*(int*)arg = itr->backwards;
+      break;
+
+    case mu_itrctl_set_direction:
+      if (!arg)
+	return EINVAL;
+      else
+	itr->backwards = !!*(int*)arg;
+      break;
+      
+    default:
+      return ENOSYS;
+    }
+  return 0;
+}
+
 int
 mu_list_get_iterator (mu_list_t list, mu_iterator_t *piterator)
 {
@@ -558,7 +676,8 @@ mu_list_get_iterator (mu_list_t list, mu_iterator_t *piterator)
   mu_iterator_set_curitem_p (iterator, curitem_p);
   mu_iterator_set_destroy (iterator, destroy);
   mu_iterator_set_dup (iterator, list_data_dup);
-
+  mu_iterator_set_itrctl (iterator, list_itrctl);
+  
   mu_iterator_attach (&list->itr, iterator);
 
   *piterator = iterator;
