@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2003, 2004, 
-   2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2007, 2008,
+   2009, 2010 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -28,16 +28,21 @@
 #include <pwd.h>
 #include <unistd.h>
 
-#ifdef HAVE_PATHS_H
-# include <paths.h>
-#endif
+#include <confpaths.h>
 
 #include <mailutils/mailbox.h>
 #include <mailutils/mutil.h>
+#include <mailutils/debug.h>
 #include <mailutils/error.h>
 #include <mailutils/errno.h>
 #include <mailutils/mu_auth.h>
 #include <mailutils/vartab.h>
+#include <mailutils/folder.h>
+#include <mailutils/auth.h>
+
+#include <mailbox0.h>
+
+char *mu_ticket_file = "~/.mu-tickets";
 
 static char *_mu_mailbox_pattern;
 
@@ -263,45 +268,30 @@ user_mailbox_name (const char *user, char **mailbox_name)
 static int
 plus_expand (const char *file, char **buf)
 {
-  char *user = NULL;
-  char *path = NULL;
   char *home;
   const char *folder_dir = mu_folder_directory ();
-  int status, len;
-  
-  if ((status = split_shortcut (file, "+=", &path, &user)))
-    return status;
+  int len;
 
-  if (!path)
-    {
-      free (user);
-      return ENOENT;
-    }
-  
-  home = get_homedir (user);
+  home = get_homedir (NULL);
   if (!home)
-    {
-      free (user);
-      free (path);
-      return ENOENT;
-    }
-
+    return ENOENT;
+  
+  file++;
+  
   if (folder_dir[0] == '/' || mu_is_proto (folder_dir))
     {
-      len = strlen (folder_dir) + strlen (path) + 2;
+      len = strlen (folder_dir) + strlen (file) + 2;
       *buf = malloc (len);
-      sprintf (*buf, "%s/%s", folder_dir, path);
+      sprintf (*buf, "%s/%s", folder_dir, file);
     }
   else
     {
-      len = strlen (home) + strlen (folder_dir) + strlen (path) + 3;
+      len = strlen (home) + strlen (folder_dir) + strlen (file) + 3;
       *buf = malloc (len);
-      sprintf (*buf, "%s/%s/%s", home, folder_dir, path);
+      sprintf (*buf, "%s/%s/%s", home, folder_dir, file);
     }
   (*buf)[len-1] = 0;
   
-  free (user);
-  free (path);
   free (home);
   return 0;
 }
@@ -328,6 +318,45 @@ percent_expand (const char *file, char **mbox)
   return status;
 }
 
+static void
+attach_auth_ticket (mu_mailbox_t mbox)
+{
+  mu_folder_t folder = NULL;
+  mu_authority_t auth = NULL;
+
+  if (mu_mailbox_get_folder (mbox, &folder) == 0
+      && mu_folder_get_authority (folder, &auth) == 0
+      && auth)
+    {
+      char *filename = mu_tilde_expansion (mu_ticket_file, "/", NULL);
+      mu_wicket_t wicket;
+      int rc;
+  
+      MU_DEBUG1 (mbox->debug, MU_DEBUG_TRACE1,
+		 "Reading user ticket file %s\n", filename);
+      if ((rc = mu_file_wicket_create (&wicket, filename)) == 0)
+	{
+	  mu_ticket_t ticket;
+      
+	  if ((rc = mu_wicket_get_ticket (wicket, NULL, &ticket)) == 0)
+	    {
+	      rc = mu_authority_set_ticket (auth, ticket);
+	      MU_DEBUG1 (mbox->debug, MU_DEBUG_TRACE1,
+			 "Retrieved and set ticket: %d\n", rc);
+	    }
+	  else
+	    MU_DEBUG1 (mbox->debug, MU_DEBUG_ERROR,
+		       "Error retrieving ticket: %s\n",
+		       mu_strerror (rc));
+	  mu_wicket_destroy (&wicket);
+	}
+      else
+	MU_DEBUG1 (mbox->debug, MU_DEBUG_ERROR,
+		   "Error creating wicket: %s\n", mu_strerror (rc));
+      free (filename);
+    }
+}
+
 /* We are trying to be smart about the location of the mail.
    mu_mailbox_create() is not doing this.
    %           --> system mailbox for the real uid
@@ -349,7 +378,10 @@ mu_mailbox_create_default (mu_mailbox_t *pmbox, const char *mail)
   if (pmbox == NULL)
     return MU_ERR_OUT_PTR_NULL;
 
-  if (mail == NULL || *mail == '\0')
+  if (mail && *mail == 0)
+    mail = NULL;
+  
+  if (mail == NULL)
     {
       if (!_mu_mailbox_pattern)
 	{
@@ -415,5 +447,8 @@ mu_mailbox_create_default (mu_mailbox_t *pmbox, const char *mail)
   
   status = mu_mailbox_create (pmbox, mbox);
   free (mbox);
+  if (status == 0)
+    attach_auth_ticket (*pmbox);
+      
   return status;
 }
