@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2004, 
-   2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2004, 2005, 2006, 2007, 2009,
+   2010 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -29,11 +29,11 @@
 #include <time.h>
 #include <unistd.h>
 #include <string.h>
-#include <ctype.h>
 #include <pwd.h>
 
 #include <message0.h>
 
+#include <mailutils/cctype.h>
 #include <mailutils/address.h>
 #include <mailutils/attribute.h>
 #include <mailutils/auth.h>
@@ -50,7 +50,7 @@
 #include <mailutils/mu_auth.h>
 #include <mailutils/nls.h>
 #include <mailutils/md5.h>
-#include <mu_umaxtostr.h>
+#include <mailutils/io.h>
 
 #define MESSAGE_MODIFIED 0x10000;
 
@@ -105,7 +105,18 @@ mu_message_destroy (mu_message_t *pmsg, void *owner)
       int destroy_lock = 0;
 
       mu_monitor_wrlock (monitor);
-      msg->ref--;
+      /* Note: msg->ref may be incremented by mu_message_ref without
+	 additional checking for its owner, therefore decrementing
+	 it must also occur independently of the owner checking. Due
+	 to this inconsistency ref may reach negative values, which
+	 is very unfortunate.
+
+	 The `owner' stuff is a leftover from older mailutils versions.
+	 There is an ongoing attempt to remove it in the stream-cleanup
+	 branch. When it is ready, it will be merged to the HEAD and this
+	 will finally resolve this issue. */
+      if (msg->ref > 0)
+	msg->ref--;
       if ((msg->owner && msg->owner == owner)
 	  || (msg->owner == NULL && msg->ref <= 0))
 	{
@@ -155,8 +166,8 @@ mu_message_destroy (mu_message_t *pmsg, void *owner)
 	     if (msg->floating_mailbox && msg->mailbox)
 	     mu_mailbox_destroy (&(msg->mailbox));
 	  */
-
-	  if (msg->ref == 0)
+	  
+	  if (msg->ref <= 0)
 	    free (msg);
 	}
       mu_monitor_unlock (monitor);
@@ -570,7 +581,8 @@ mu_message_get_uid (mu_message_t msg, size_t *puid)
 }
 
 int
-mu_message_get_uidl (mu_message_t msg, char *buffer, size_t buflen, size_t *pwriten)
+mu_message_get_uidl (mu_message_t msg, char *buffer, size_t buflen,
+		     size_t *pwriten)
 {
   mu_header_t header = NULL;
   size_t n = 0;
@@ -591,29 +603,8 @@ mu_message_get_uidl (mu_message_t msg, char *buffer, size_t buflen, size_t *pwri
   /* Be compatible with Qpopper ? qppoper saves the UIDL in "X-UIDL".
      We generate a chksum and save it in the header.  */
   mu_message_get_header (msg, &header);
-  status = mu_header_get_value (header, "X-UIDL", buffer, buflen, &n);
-  if (status == 0 && n > 0)
-    {
-      /* FIXME: Is this necessary ? I did not see so far a x-uidl message
-	 that broken i.e.
-	 X-UIDL:  <abaceekeke\n
-	       jakdkjaja>
-      */
-      /* We need to collapse the header if it was mutiline.  e points to the
-	 last char meaning in a C string that's '\0', s to the start. We also
-	 remove the spesky '<' '>' if they are around.  */
-      char *s, *e;
-      for (s = buffer, e = buffer + n; s <= e; s++)
-	{
-	  if (isspace ((unsigned char)*s) || *s == '<' || *s == '>')
-	    {
-	      memmove (s, s + 1, e - (s + 1));
-	      e -= 1;
-	      *e = '\0';
-	    }
-	}
-    }
-  else
+  status = mu_header_get_value_unfold (header, "X-UIDL", buffer, buflen, &n);
+  if (status != 0 || n == 0)
     {
       size_t uid = 0;
       struct mu_md5_ctx md5context;
@@ -638,12 +629,13 @@ mu_message_get_uidl (mu_message_t msg, char *buffer, size_t buflen, size_t *pwri
 	sprintf (tmp, "%02x", md5digest[n]);
       *tmp = '\0';
       /* POP3 rfc says that an UID should not be longer than 70.  */
-      snprintf (buf + 32, 70, ".%lu.%s", (unsigned long)time (NULL), 
-                mu_umaxtostr (0, uid));
+      snprintf (buf + 32, 70, ".%lu.%lu", (unsigned long)time (NULL), 
+                (unsigned long) uid);
 
       mu_header_set_value (header, "X-UIDL", buf, 1);
       buflen--; /* leave space for the NULL.  */
       strncpy (buffer, buf, buflen)[buflen] = '\0';
+      status = 0;
     }
   return status;
 }
@@ -1110,9 +1102,9 @@ message_body_read (mu_stream_t stream,  char *buffer, size_t n, mu_off_t off,
 }
 
 int
-mu_message_save_to_mailbox (mu_message_t msg, mu_ticket_t ticket,
+mu_message_save_to_mailbox (mu_message_t msg, 
                             mu_debug_t debug,
-			    const char *toname)
+			    const char *toname, int perms)
 {
   int rc = 0;
   mu_mailbox_t to = 0;
@@ -1128,27 +1120,9 @@ mu_message_save_to_mailbox (mu_message_t msg, mu_ticket_t ticket,
   if (debug && (rc = mu_mailbox_set_debug (to, debug)))
 	goto end;
 
-  if (ticket)
-    {
-      mu_folder_t folder = NULL;
-
-      if ((rc = mu_mailbox_get_folder (to, &folder)))
-	goto end;
-
-      /* FIXME: not all mailboxes have folders, thus this hack. */
-      if (folder)
-	{
-	  mu_authority_t auth = NULL;
-	  if ((rc = mu_folder_get_authority (folder, &auth)))
-	    goto end;
-
-	  /* FIXME: not all folders have authentication, thus this hack. */
-	  if (auth && (rc = mu_authority_set_ticket (auth, ticket)))
-	    goto end;
-	}
-    }
-
-  if ((rc = mu_mailbox_open (to, MU_STREAM_WRITE | MU_STREAM_CREAT)))
+  if ((rc = mu_mailbox_open (to,
+			     MU_STREAM_WRITE | MU_STREAM_CREAT
+			     | (perms & MU_STREAM_IMASK))))
     {
       MU_DEBUG2 (debug, MU_DEBUG_ERROR,
 		 "mu_mailbox_open (%s) failed: %s\n", toname,
