@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003,
-   2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2007, 2009,
+   2010 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -33,8 +33,8 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <sys/param.h>
-#include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/wait.h>
 #include <sys/select.h>
@@ -60,6 +60,9 @@
 #include <mailutils/filter.h>
 #include <mailutils/sql.h>
 #include <mailutils/url.h>
+#include <mailutils/io.h>
+#include <mailutils/cctype.h>
+#include <mailutils/cstr.h>
 
 #include <registrar0.h>
 
@@ -72,10 +75,10 @@ mu_hex2ul (char hex)
    return hex - '0';
 
   if (hex >= 'a' && hex <= 'z')
-    return hex - 'a';
+    return hex - 'a' + 10;
 
   if (hex >= 'A' && hex <= 'Z')
-    return hex - 'A';
+    return hex - 'A' + 10;
 
   return -1;
 }
@@ -288,37 +291,6 @@ mu_cpystr (char *dst, const char *src, size_t size)
   return len;
 }
 
-/* General retrieve stack support: */
-
-void
-mu_register_retriever (mu_list_t *pflist, mu_retrieve_fp fun)
-{
-  if (!*pflist && mu_list_create (pflist))
-    return;
-  mu_list_append (*pflist, fun);
-}
-
-void *
-mu_retrieve (mu_list_t flist, void *data)
-{
-  void *p = NULL;
-  mu_iterator_t itr;
-
-  if (mu_list_get_iterator (flist, &itr) == 0)
-    {
-      mu_retrieve_fp fun;
-      for (mu_iterator_first (itr); !p && !mu_iterator_is_done (itr);
-	   mu_iterator_next (itr))
-	{
-	  mu_iterator_current (itr, (void **)&fun);
-	  p = (*fun) (data);
-	}
-
-      mu_iterator_destroy (&itr);
-    }
-  return p;
-}
-
 int
 mu_get_host_name (char **host)
 {
@@ -411,6 +383,7 @@ mu_set_user_email_domain (const char *domain)
   return 0;
 }
 
+/* FIXME: must be called _sget_ */
 int
 mu_get_user_email_domain (const char **domain)
 {
@@ -427,6 +400,24 @@ mu_get_user_email_domain (const char **domain)
   return 0;
 }
 
+int
+mu_aget_user_email_domain (char **pdomain)
+{
+  const char *domain;
+  int status = mu_get_user_email_domain (&domain);
+  if (status)
+    return status;
+  if (domain == NULL)
+    *pdomain = NULL;
+  else
+    {
+      *pdomain = strdup (domain);
+      if (*pdomain == NULL)
+	return ENOMEM;
+    }
+  return 0;
+}
+
 /* Note: allocates memory */
 char *
 mu_get_user_email (const char *name)
@@ -495,10 +486,9 @@ mu_get_user_email (const char *name)
          /home/user/../smith   -->   /home/smith
 	 /home/user/../..      -->   /
 
-   FIXME: delim is superfluous. The function deals with unix filesystem
-   paths, so delim should be always "/" */
+*/
 char *
-mu_normalize_path (char *path, const char *delim)
+mu_normalize_path (char *path)
 {
   int len;
   char *p;
@@ -513,21 +503,21 @@ mu_normalize_path (char *path, const char *delim)
     return path;
 
   /* delete trailing delimiter if any */
-  if (len && path[len-1] == delim[0])
+  if (len && path[len-1] == '/')
     path[len-1] = 0;
 
   /* Eliminate any /../ */
   for (p = strchr (path, '.'); p; p = strchr (p, '.'))
     {
-      if (p > path && p[-1] == delim[0])
+      if (p > path && p[-1] == '/')
 	{
-	  if (p[1] == '.' && (p[2] == 0 || p[2] == delim[0]))
+	  if (p[1] == '.' && (p[2] == 0 || p[2] == '/'))
 	    /* found */
 	    {
 	      char *q, *s;
 
 	      /* Find previous delimiter */
-	      for (q = p-2; *q != delim[0] && q >= path; q--)
+	      for (q = p-2; *q != '/' && q >= path; q--)
 		;
 
 	      if (q < path)
@@ -546,7 +536,7 @@ mu_normalize_path (char *path, const char *delim)
 
   if (path[0] == 0)
     {
-      path[0] = delim[0];
+      path[0] = '/';
       path[1] = 0;
     }
 
@@ -593,7 +583,7 @@ mu_tempfile (const char *tmpdir, char **namep)
 
   if (fd == -1)
     {
-      mu_error (_("Cannot open temporary file: %s"), strerror (errno));
+      mu_error (_("cannot open temporary file: %s"), mu_strerror (errno));
       free (filename);
       return -1;
     }
@@ -800,10 +790,10 @@ mu_unroll_symlink (char *out, size_t outsz, const char *in)
 char *
 mu_expand_path_pattern (const char *pattern, const char *username)
 {
-  const char *p, *startp;
+  const char *p;
   char *q;
   char *path;
-  int len = 0;
+  size_t len = 0;
   struct mu_auth_data *auth = NULL;
   
   for (p = pattern; *p; p++)
@@ -850,39 +840,46 @@ mu_expand_path_pattern (const char *pattern, const char *username)
   if (!path)
     return NULL;
 
-  startp = pattern;
+  p = pattern;
   q = path;
-  while (*startp && (p = strchr (startp, '%')) != NULL)
+  while (*p)
     {
-      memcpy (q, startp, p - startp);
-      q += p - startp;
-      switch (*++p)
+      size_t off = strcspn (p, "~%");
+      memcpy (q, p, off);
+      q += off;
+      p += off;
+      if (*p == '~')
 	{
-	case 'u':
-	  strcpy (q, username);
-	  q += strlen (username);
-	  break;
-	  
-	case 'h':
 	  strcpy (q, auth->dir);
 	  q += strlen (auth->dir);
-	  break;
+	  p++;
+	}
+      else if (*p == '%')
+	{
+	  switch (*++p)
+	    {
+	    case 'u':
+	      strcpy (q, username);
+	      q += strlen (username);
+	      break;
+	    
+	    case 'h':
+	      strcpy (q, auth->dir);
+	      q += strlen (auth->dir);
+	      break;
 	  
-	case '%':
-	  *q++ = '%';
-	  break;
+	    case '%':
+	      *q++ = '%';
+	      break;
 	  
-	default:
-	  *q++ = '%';
-	  *q++ = *p;
+	    default:
+	      *q++ = '%';
+	      *q++ = *p;
+	    }
+	  p++;
 	}
-      startp = p + 1;
-    }
-  if (*startp)
-    {
-      strcpy (q, startp);
-      q += strlen (startp);
     }
+
   *q = 0;
   if (auth)
     mu_auth_data_free (auth);
@@ -1013,14 +1010,14 @@ mu_rfc2822_msg_id (int subpart, char **pval)
     {
       struct timeval tv;
       gettimeofday (&tv, NULL);
-      asprintf (&p, "<%s.%lu.%d@%s>",
-		date,
-		(unsigned long) getpid (),
-		subpart,
-		host);
+      mu_asprintf (&p, "<%s.%lu.%d@%s>",
+		   date,
+		   (unsigned long) getpid (),
+		   subpart,
+		   host);
     }
   else
-    asprintf (&p, "<%s.%lu@%s>", date, (unsigned long) getpid (), host);
+    mu_asprintf (&p, "<%s.%lu@%s>", date, (unsigned long) getpid (), host);
   free (host);
   *pval = p;
   return 0;
@@ -1096,7 +1093,7 @@ mu_strcasestr (const char *a_haystack, const char *a_needle)
     *needle = (unsigned char*) a_needle;
   register unsigned int b, c;
 
-#define U(c) toupper (c)
+#define U(c) mu_toupper (c)
   if ((b = U (*needle)))
     {
       haystack--;		
@@ -1277,10 +1274,10 @@ mu_fd_wait (int fd, int *pflags, struct timeval *tvp)
       if (tvp)
 	{
 	  struct timeval tv = *tvp; 
-	  rc = select (fd + 1, &rdset, &wrset, NULL, &tv);
+	  rc = select (fd + 1, &rdset, &wrset, &exset, &tv);
 	}
       else
-	rc = select (fd + 1, &rdset, &wrset, NULL, NULL);
+	rc = select (fd + 1, &rdset, &wrset, &exset, NULL);
     }
   while (rc == -1 && errno == EINTR);
 
@@ -1328,22 +1325,19 @@ mu_decode_filter (mu_stream_t *pfilter, mu_stream_t input,
   if (status)
     return status;
 
-  if (fromcode && tocode && strcasecmp (fromcode, tocode))
+  if (fromcode && tocode && mu_c_strcasecmp (fromcode, tocode))
     {
       mu_stream_t cvt;
       status = mu_filter_iconv_create (&cvt, filter, fromcode, tocode,
-				    MU_STREAM_NO_CLOSE,
-				    mu_default_fallback_mode);
+				       MU_STREAM_NO_CLOSE,
+				       mu_default_fallback_mode);
       if (status == 0)
 	{
 	  if (mu_stream_open (cvt))
 	    mu_stream_destroy (&cvt, mu_stream_get_owner (cvt));
 	  else
 	    {
-	      int flags;
-	      mu_stream_get_flags (cvt, &flags);
-	      flags &= ~MU_STREAM_NO_CLOSE;
-	      mu_stream_set_flags (cvt, flags);
+	      mu_stream_clr_flags (cvt, MU_STREAM_NO_CLOSE);
 	      filter = cvt;
 	    }
 	}
@@ -1355,6 +1349,8 @@ mu_decode_filter (mu_stream_t *pfilter, mu_stream_t input,
 int
 mu_is_proto (const char *p)
 {
+  if (*p == '|')
+    return 1;
   for (; *p && *p != '/'; p++)
     if (*p == ':')
       return 1;
@@ -1470,3 +1466,26 @@ mu_sql_decode_password_type (const char *arg, enum mu_password_type *t)
   return 0;
 }
 
+int
+mu_stream_flags_to_mode (int flags, int isdir)
+{
+  int mode = 0;
+  if (flags & MU_STREAM_IRGRP)
+    mode |= S_IRGRP;
+  if (flags & MU_STREAM_IWGRP)
+    mode |= S_IWGRP;
+  if (flags & MU_STREAM_IROTH)
+    mode |= S_IROTH;
+  if (flags & MU_STREAM_IWOTH)
+    mode |= S_IWOTH;
+
+  if (isdir)
+    {
+      if (mode & (S_IRGRP|S_IWGRP))
+	mode |= S_IXGRP;
+      if (mode & (S_IROTH|S_IWOTH))
+	mode |= S_IXOTH;
+    }
+  
+  return mode;
+}
