@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2004, 2005, 2006, 2007, 
-   2009 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2004, 2005, 2006, 2007, 2009, 2010 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -42,23 +42,27 @@ realloc_buffer (char **bufp, size_t *bufsizep, size_t incr)
   return 0;
 }
 
-static char *
-getword (const char **pstr, int delim)
+int
+getword (char **pret, const char **pstr, int delim)
 {
   size_t len;
   char *ret;
   const char *start = *pstr;
   const char *end = strchr (start, delim);
+
+  free (*pret);
+  *pret = NULL;
   if (!end)
-    return NULL;
+    return MU_ERR_BAD_2047_INPUT;
   len = end - start;
   ret = malloc (len + 1);
   if (!ret)
-    return NULL;
+    return ENOMEM;
   memcpy (ret, start, len);
   ret[len] = 0;
   *pstr = end + 1;
-  return ret;
+  *pret = ret;
+  return 0;
 }
     
 int
@@ -92,7 +96,7 @@ mu_rfc2047_decode (const char *tocode, const char *input, char **ptostr)
      }                                           \
  } while (0) 
   
-  if (!tocode || !input)
+  if (!input)
     return EINVAL;
   if (!ptostr)
     return MU_ERR_OUT_PTR_NULL;
@@ -117,24 +121,23 @@ mu_rfc2047_decode (const char *tocode, const char *input, char **ptostr)
 	  const char *filter_type = NULL;
 	  size_t nbytes = 0, size;
 	  const char *sp = fromstr + 2;
-
-	  fromcode = getword (&sp, '?');
-	  encoding_type = getword (&sp, '?');
-	  encoded_text = getword (&sp, '?');
+	  char tmp[128];
+	  
+	  status = getword (&fromcode, &sp, '?');
+	  if (status)
+	    break;
+	  status = getword (&encoding_type, &sp, '?');
+	  if (status)
+	    break;
+	  status = getword (&encoded_text, &sp, '?');
+	  if (status)
+	    break;
 	  if (sp == NULL || sp[0] != '=')
 	    {
 	      status = MU_ERR_BAD_2047_INPUT;
 	      break;
 	    }
       
-	  if (fromcode == NULL
-	      || encoding_type == NULL
-	      || encoded_text == NULL)
-	    {
-	      status = MU_ERR_BAD_2047_INPUT;
-	      break;
-	    }
-
 	  size = strlen (encoded_text);
 
 	  switch (encoding_type[0])
@@ -164,11 +167,14 @@ mu_rfc2047_decode (const char *tocode, const char *input, char **ptostr)
 	  if (status != 0)
 	    break;
 
-	  while (mu_stream_sequential_read (filter, buffer + bufpos,
-					 bufsize - bufpos,
-					 &nbytes) == 0
+	  while (mu_stream_sequential_read (filter, tmp, sizeof (tmp),
+					    &nbytes) == 0
 		 && nbytes)
-	    bufpos += nbytes;
+	    {
+	      CHKBUF (nbytes);
+	      memcpy (buffer + bufpos, tmp, nbytes);
+	      bufpos += nbytes;
+	    }
 
 	  mu_stream_close (filter);
 	  mu_stream_destroy (&filter, mu_stream_get_owner (filter));
@@ -188,18 +194,18 @@ mu_rfc2047_decode (const char *tocode, const char *input, char **ptostr)
 	    {
 	      if (--run_count)
 		{
-		  CHKBUF(run_count);
+		  CHKBUF (run_count);
 		  memcpy (buffer + bufpos, fromstr - run_count, run_count);
 		  bufpos += run_count;
 		  run_count = 0;
 		}
-	      CHKBUF(1);
+	      CHKBUF (1);
 	      buffer[bufpos++] = *fromstr++;
 	    }
 	}
       else
 	{
-	  CHKBUF(1);
+	  CHKBUF (1);
 	  buffer[bufpos++] = *fromstr++;
 	}
     }
@@ -207,19 +213,22 @@ mu_rfc2047_decode (const char *tocode, const char *input, char **ptostr)
   if (*fromstr)
     {
       size_t len = strlen (fromstr);
-      CHKBUF(len);
-      memcpy (buffer + bufpos, fromstr, strlen (fromstr));
-      bufpos += strlen (fromstr);
+      CHKBUF (len);
+      memcpy (buffer + bufpos, fromstr, len);
+      bufpos += len;
     }
 
-  CHKBUF(1);
+  CHKBUF (1);
   buffer[bufpos++] = 0;
   
   free (fromcode);
   free (encoding_type);
   free (encoded_text);
-  
-  *ptostr = realloc (buffer, bufpos);
+
+  if (status)
+    free (buffer);
+  else
+    *ptostr = realloc (buffer, bufpos);
   return status;
 }
 
