@@ -1,10 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003,
-   2004 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008,
+   2009, 2010 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -27,6 +27,7 @@
 #endif
 
 #include <mailutils/mailutils.h>
+#include "mailutils/libargp.h"
 
 static int messages_count (const char *);
 
@@ -38,18 +39,17 @@ static struct argp_option options[] = {
   { NULL,         0, NULL,  0,
     /* TRANSLATORS: 'messages' is a program name. Do not translate it! */
     N_("messages specific switches:"), 0},
-  {"quiet",	'q',	0,	0,	N_("Only display number of messages")},
-  {"silent",	's',	0,	0,	N_("Same as -q")},
+  {"quiet",	'q',	NULL,	0,	N_("only display number of messages")},
+  {"silent",	's',	NULL,	OPTION_ALIAS, NULL },
   { 0 }
 };
 
 static const char *argp_capa[] = {
   "common",
+  "debug",
   "license",
   "mailbox",
-#ifdef WITH_TLS
-  "tls",
-#endif
+  "locking",
   NULL
 };
 
@@ -72,6 +72,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case 's':
       silent = 1;
       break;
+      
     case ARGP_KEY_ARG:
       args->argv = realloc (args->argv,
 			    sizeof (char *) * (state->arg_num + 2));
@@ -79,6 +80,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       args->argv[state->arg_num + 1] = NULL;
       args->argc++;
       break;
+      
     default:
       return ARGP_ERR_UNKNOWN;
     }
@@ -102,17 +104,18 @@ main (int argc, char **argv)
   struct arguments args = {0, NULL};
 
   /* Native Language Support */
-  mu_init_nls ();
-
-  mu_argp_init (program_version, NULL);
-#ifdef WITH_TLS
-  mu_tls_init_client_argp ();
-#endif
-  mu_argp_parse (&argp, &argc, &argv, 0, argp_capa, NULL, &args);
+  MU_APP_INIT_NLS ();
 
   /* register the formats.  */
   mu_register_all_mbox_formats ();
 
+#ifdef WITH_TLS
+  mu_gocs_register ("tls", mu_tls_module_init);
+#endif
+  mu_argp_init (program_version, NULL);
+  if (mu_app_init (&argp, argp_capa, NULL, argc, argv, 0, NULL, &args))
+    exit (1);
+
   if (args.argc < 1 && messages_count (NULL) < 0)
     err = 1;
   else if (args.argc >= 1)
@@ -130,55 +133,55 @@ main (int argc, char **argv)
 static int
 messages_count (const char *box)
 {
-  mailbox_t mbox;
-  url_t url = NULL;
+  mu_mailbox_t mbox;
+  mu_url_t url = NULL;
   size_t count;
   int status = 0;
 
-  status =  mailbox_create_default (&mbox, box);
+  status =  mu_mailbox_create_default (&mbox, box);
   if (status != 0)
     {
       if (box)
-	mu_error (_("Could not create mailbox `%s': %s"),
+	mu_error (_("could not create mailbox `%s': %s"),
 		  box, mu_strerror (status));
       else
-	mu_error (_("Could not create default mailbox: %s"),
+	mu_error (_("could not create default mailbox: %s"),
 		  mu_strerror (status));
       return -1;
     }
 
-  mailbox_get_url (mbox, &url);
-  box = url_to_string (url);
+  mu_mailbox_get_url (mbox, &url);
+  box = mu_url_to_string (url);
 
-  status =  mailbox_open (mbox, MU_STREAM_READ);
+  status =  mu_mailbox_open (mbox, MU_STREAM_READ);
   if (status != 0)
     {
-      mu_error (_("Could not open mailbox `%s': %s"),
+      mu_error (_("could not open mailbox `%s': %s"),
 		box, mu_strerror (status));
       return -1;
     }
 
-  status = mailbox_messages_count (mbox, &count);
+  status = mu_mailbox_messages_count (mbox, &count);
   if (status != 0)
     {
-      mu_error (_("Could not count messages in mailbox `%s': %s"),
+      mu_error (_("could not count messages in mailbox `%s': %s"),
 		box, mu_strerror (status));
       return -1;
     }
 
   if (silent)
-    printf ("%d\n", count);
+    printf ("%lu\n", (unsigned long) count);
   else
-    printf (_("Number of messages in %s: %d\n"), box, count);
+    printf (_("Number of messages in %s: %lu\n"), box, (unsigned long) count);
 
-  status = mailbox_close (mbox);
+  status = mu_mailbox_close (mbox);
   if (status != 0)
     {
-      mu_error (_("Could not close `%s': %s"),
+      mu_error (_("could not close `%s': %s"),
 		box, mu_strerror (status));
       return -1;
     }
 
-  mailbox_destroy (&mbox);
+  mu_mailbox_destroy (&mbox);
   return count;
 }
