@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 
-   2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2007, 2008,
+   2009, 2010 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -42,47 +42,47 @@ static char args_doc[] = N_("[action] [msg]");
 static struct argp_option options[] = {
   {N_("Actions are:"), 0, 0, OPTION_DOC, NULL, 0 },
   {"print", ARG_PRINT, NULL, 0,
-   N_("List the folders (default)"), 1 },
+   N_("list the folders (default)"), 1 },
   {"list",  ARG_LIST,  NULL, 0,
-   N_("List the contents of the folder stack"), 1},
+   N_("list the contents of the folder stack"), 1},
   {"pack",  ARG_PACK,  N_("NUMBER"), OPTION_ARG_OPTIONAL,
-   N_("Remove holes in message numbering. Begin numbering from NUMBER (default: first message number)"), 1},
+   N_("remove holes in message numbering, begin numbering from NUMBER (default: first message number)"), 1},
   {"push",  ARG_PUSH,  N_("FOLDER"), OPTION_ARG_OPTIONAL,
-    N_("Push the folder on the folder stack. If FOLDER is specified, it is pushed. "
+    N_("push the folder on the folder stack. If FOLDER is specified, it is pushed. "
        "Otherwise, if a folder is given in the command line (via + or --folder), "
        "it is pushed on stack. Otherwise, the current folder and the top of the folder "
        "stack are exchanged"), 1},
   {"pop",   ARG_POP,    NULL, 0,
-   N_("Pop the folder off the folder stack"), 1},
+   N_("pop the folder off the folder stack"), 1},
   
   {N_("Options are:"), 0, 0, OPTION_DOC, NULL, 2 },
   
   {"folder", ARG_FOLDER, N_("FOLDER"), 0,
-   N_("Specify folder to operate upon"), 3},
+   N_("specify folder to operate upon"), 3},
   {"all",    ARG_ALL,    NULL, 0,
-   N_("List all folders"), 3},
+   N_("list all folders"), 3},
   {"create", ARG_CREATE, N_("BOOL"), OPTION_ARG_OPTIONAL, 
-    N_("Create non-existing folders"), 3},
+    N_("create non-existing folders"), 3},
   {"nocreate", ARG_NOCREATE, NULL, OPTION_HIDDEN, ""},
   {"fast",   ARG_FAST, N_("BOOL"), OPTION_ARG_OPTIONAL, 
-    N_("List only the folder names"), 3},
+    N_("list only the folder names"), 3},
   {"nofast", ARG_NOFAST, NULL, OPTION_HIDDEN, ""},
   {"header", ARG_HEADER, N_("BOOL"), OPTION_ARG_OPTIONAL, 
-    N_("Print the header line"), 3},
+    N_("print the header line"), 3},
   {"noheader", ARG_NOHEADER, NULL, OPTION_HIDDEN, ""},
   {"recurse",ARG_RECURSIVE, N_("BOOL"), OPTION_ARG_OPTIONAL,
-    N_("Scan folders recursively"), 3},
+    N_("scan folders recursively"), 3},
   {"norecurse", ARG_NORECURSIVE, NULL, OPTION_HIDDEN, ""},
   {"total",  ARG_TOTAL, N_("BOOL"), OPTION_ARG_OPTIONAL, 
-    N_("Output the total statistics"), 3},
+    N_("output the total statistics"), 3},
   {"nototal", ARG_NOTOTAL, NULL, OPTION_HIDDEN, ""},
   {"verbose", ARG_VERBOSE, NULL, 0,
-   N_("Verbosely list actions taken"), 3},
+   N_("verbosely list actions taken"), 3},
   {"dry-run", ARG_DRY_RUN, NULL, 0,
-   N_("Do nothing, print what would be done (with --pack)"), 3},
+   N_("do nothing, print what would be done (with --pack)"), 3},
    
   {"license", ARG_LICENSE, 0,      0,
-   N_("Display software license"), -1},
+   N_("display software license"), -1},
 
   {NULL},
 };
@@ -136,7 +136,7 @@ size_t max_depth = 1;  /* Maximum recursion depth (0 means infinity) */
 #define OPTION_IS_SET(opt) ((opt) == -1 ? show_all : opt)
 
 static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+opt_handler (int key, char *arg, struct argp_state *state)
 {
   switch (key)
     {
@@ -151,7 +151,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
 	  char *p;
 	  pack_start = strtoul (arg, &p, 10);
 	  if (*p)
-	    argp_error (state, _("Invalid number"));
+	    argp_error (state, _("invalid number"));
 	}
       break;
       
@@ -234,7 +234,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     default:
-      return 1;
+      return ARGP_ERR_UNKNOWN;
     }
   return 0;
 }
@@ -327,7 +327,7 @@ _scan (const char *name, size_t depth)
 
   if (!dir)
     {
-      mu_error (_("Cannot scan folder %s: %s"), name, strerror (errno));
+      mu_error (_("cannot scan folder %s: %s"), name, strerror (errno));
       return;
     }
 
@@ -362,7 +362,7 @@ _scan (const char *name, size_t depth)
 	{
 	  asprintf (&p, "%s/%s", name, entry->d_name);
 	  if (stat (p, &st) < 0)
-	    mu_error (_("Cannot stat %s: %s"), p, strerror (errno));
+	    mu_diag_funcall (MU_DIAG_ERROR, "stat", p, errno);
 	  else if (S_ISDIR (st.st_mode))
 	    {
 	      info.others++;
@@ -531,15 +531,16 @@ action_list ()
 static void
 get_stack (int *pc, char ***pv)
 {
+  int status;
   const char *stack = mh_global_context_get ("Folder-Stack", NULL);
   if (!stack)
     {
       *pc = 0;
       *pv = NULL;
     }
-  else if (mu_argcv_get (stack, NULL, "#", pc, pv))
+  else if ((status = mu_argcv_get (stack, NULL, "#", pc, pv)) != 0)
     {
-      mu_error (_("Cannot split line %s"), stack);
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_argcv_get", stack, status);
       exit (1);
     }
 }
@@ -722,7 +723,7 @@ roll_back (const char *folder_name, struct pack_tab *pack_tab, size_t i)
     return;
   
   start = i - 1;
-  mu_error (_("Rolling back changes..."));
+  mu_error (_("rolling back changes..."));
   while (--i >= 0)
     if (pack_rename (pack_tab + i, 1))
       {
@@ -737,7 +738,7 @@ roll_back (const char *folder_name, struct pack_tab *pack_tab, size_t i)
 	mu_error (_("You will have to fix it manually."));
 	exit (1);
       }
-  mu_error (_("Folder `%s' restored successfully"), folder_name);
+  mu_error (_("folder `%s' restored successfully"), folder_name);
 }
 
 struct fixup_data
@@ -846,7 +847,7 @@ action_pack ()
   /* Allocate pack table */
   if (mu_mailbox_messages_count (mbox, &count))
     {
-      mu_error (_("Cannot read input mailbox: %s"), mu_strerror (errno));
+      mu_error (_("cannot read input mailbox: %s"), mu_strerror (errno));
       return 1;
     }
   pack_tab = xcalloc (count, sizeof pack_tab[0]); /* Never freed. No use to
@@ -862,7 +863,8 @@ action_pack ()
       status = mu_mailbox_get_message (mbox, i + 1, &msg);
       if (status)
 	{
-	  mu_error (_("%d: cannot get message: %s"), i, mu_strerror (status));
+	  mu_error (_("%lu: cannot get message: %s"),
+		    (unsigned long) i, mu_strerror (status));
 	  return 1;
 	}
       mh_message_number (msg, &pack_tab[i].orig);
@@ -935,15 +937,15 @@ main (int argc, char **argv)
   mh_msgset_t msgset;
 
   /* Native Language Support */
-  mu_init_nls ();
+  MU_APP_INIT_NLS ();
 
-  mu_argp_init (program_version, NULL);
+  mh_argp_init (program_version);
   mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, &index);
 
   /* If  folder  is invoked by a name ending with "s" (e.g.,  folders),
      `-all'  is  assumed */
-  if (program_invocation_short_name[strlen (program_invocation_short_name) - 1] == 's')
+  if (mu_program_name[strlen (mu_program_name) - 1] == 's')
     show_all = 1;
   
   if (has_folder)
@@ -968,7 +970,7 @@ main (int argc, char **argv)
     }
   else if (argc - index > 1)
     {
-      mu_error (_("Too many arguments"));
+      mu_error (_("too many arguments"));
       exit (1);
     }
   
