@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2004, 
-   2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2004, 2005, 2006, 2007, 2009,
+   2010 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -111,7 +111,7 @@ mh_context_read (mh_context_t *ctx)
     {
       char *q;
 
-      for (q = buf; *q && isspace (*q); q++)
+      for (q = buf; *q && mu_isspace (*q); q++)
 	;
       if (!*q || *q == '#')
 	continue;
@@ -141,7 +141,7 @@ mh_context_write (mh_context_t *ctx)
   fp = fopen (ctx->name, "w");
   if (!fp)
     {
-      mu_error (_("Cannot open context file %s: %s"),
+      mu_error (_("cannot open context file %s: %s"),
 		ctx->name, strerror (errno));
       return MU_ERR_FAILURE;
     }
@@ -180,7 +180,7 @@ mh_context_set_value (mh_context_t *ctx, const char *name, const char *value)
       int rc;
       if ((rc = mu_header_create (&ctx->header, NULL, 0, NULL)) != 0)
 	{
-	  mu_error (_("Cannot create context %s: %s"),
+	  mu_error (_("cannot create context %s: %s"),
 		    ctx->name,
 		    mu_strerror (rc));
 	  return 1;
