@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002, 2005, 2007  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2005, 2007, 2009, 2010 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,8 +17,9 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301 USA */
 
-#include <mailutils/argp.h>
 #include <mailutils/nls.h>
+#include "argp.h"
+#include "mailutils/libargp.h"
 
 #define MH_OPT_BOOL 1
 #define MH_OPT_ARG  2
@@ -33,7 +35,7 @@ struct mh_option
 struct mh_argp_data
 {
   struct mh_option *mh_option;
-  int (*handler)();
+  argp_parser_t handler;
   int errind;
   void *closure;
   char *doc;
@@ -53,7 +55,8 @@ enum mh_arg {
   ARG_BUILD,		
   ARG_CC,	        
   ARG_CFLAGS,		
-  ARG_CHANGECUR,		
+  ARG_CHANGECUR,
+  ARG_CHARSET,
   ARG_CHECK,
   ARG_CHUNKSIZE,
   ARG_CLEAR,		
@@ -181,6 +184,7 @@ enum mh_arg {
   ARG_ZERO
 };
 
+void mh_argp_init (const char *vers);
 void mh_argv_preproc (int argc, char **argv, struct mh_argp_data *data);
 int mh_getopt (int argc, char **argv, struct mh_option *mh_opt, const char *doc);
 int mh_argp_parse (int *argc, char **argv[],
@@ -188,6 +192,11 @@ int mh_argp_parse (int *argc, char **argv[],
 		   struct argp_option *option,
 		   struct mh_option *mh_option,
 		   char *argp_doc, char *doc,
-		   int (*handler)(), void *closure, int *index);
+		   argp_parser_t handler,
+		   void *closure, int *index);
+
 void mh_help (struct mh_option *mh_option, const char *doc);
 void mh_license (const char *name);
+
+void mh_opt_notimpl (const char *name);
+void mh_opt_notimpl_warning (const char *name);
