@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2007, 2008, 2009, 2010 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -28,11 +29,11 @@ static char args_doc[] = N_("[components]");
 /* GNU options */
 static struct argp_option options[] = {
   {"all",  ARG_ALL, NULL, 0,
-   N_("Display all components from the MH profile. All other arguments are ignored")},
+   N_("display all components from the MH profile. All other arguments are ignored")},
   {"component", ARG_COMPONENT, N_("BOOL"),   OPTION_ARG_OPTIONAL,
-   N_("Always display the component name") },
+   N_("always display the component name") },
   {"license", ARG_LICENSE, 0,      0,
-   N_("Display software license"), -1},
+   N_("display software license"), -1},
   { 0 }
 };
 
@@ -46,8 +47,8 @@ struct mh_option mh_option[] = {
 static int display_all;
 static int display_comp_name = -1;
 
-static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+static error_t
+opt_handler (int key, char *arg, struct argp_state *state)
 {
   switch (key)
     {
@@ -64,7 +65,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
 
     default:
-      return 1;
+      return ARGP_ERR_UNKNOWN;
     }
   return 0;
 }
@@ -82,14 +83,14 @@ char *
 mhparam_defval (char *comp)
 {
   int i;
-  for (i = 0; i < sizeof(defvaltab)/sizeof(defvaltab[0]); i++)
-    if (strcasecmp(defvaltab[i].comp, comp) == 0)
+  for (i = 0; i < sizeof (defvaltab) / sizeof (defvaltab[0]); i++)
+    if (mu_c_strcasecmp (defvaltab[i].comp, comp) == 0)
       return defvaltab[i].val;
   return NULL;
 }
 
 int
-mhparam_iterator (char *comp, char *value, char *data ARG_UNUSED)
+mhparam_iterator (const char *comp, const char *value, void *data MU_ARG_UNUSED)
 {
   if (display_comp_name)
     printf("%s:\t", comp);
@@ -102,7 +103,7 @@ mhparam (char *comp)
 {
   if (comp)
     {
-      char *val = mh_global_profile_get (comp, NULL);
+      const char *val = mh_global_profile_get (comp, NULL);
       if (!val)
 	val = mhparam_defval (comp);
       if (!val)
@@ -123,9 +124,9 @@ main (int argc, char **argv)
   int index;
   
   /* Native Language Support */
-  mu_init_nls ();
+  MU_APP_INIT_NLS ();
 
-  mu_argp_init (program_version, NULL);
+  mh_argp_init (program_version);
   mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, &index);
 
