@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002, 2003, 2004, 2005, 2006, 
-   2007 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+   Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -35,20 +35,20 @@ static char args_doc[] = N_("messages folder [folder...]");
 /* GNU options */
 static struct argp_option options[] = {
   {"folder",  ARG_FOLDER, N_("FOLDER"), 0,
-   N_("Specify folder to operate upon")},
+   N_("specify folder to operate upon")},
   {"draft",   ARG_DRAFT, NULL, 0,
-   N_("Use <mh-dir>/draft as the source message")},
+   N_("use <mh-dir>/draft as the source message")},
   {"copy",    ARG_LINK, N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Preserve the source folder copy")},
+   N_("preserve the source folder copy")},
   {"link",    0, NULL, OPTION_ALIAS, NULL},
   {"preserve", ARG_PRESERVE, N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("* Try to preserve message sequence numbers")},
+   N_("* try to preserve message sequence numbers")},
   {"source", ARG_SOURCE, N_("FOLDER"), 0,
-   N_("Specify source folder. FOLDER will become the current folder after the program exits")},
+   N_("specify source folder; it will become the current folder after the program exits")},
   {"src", 0, NULL, OPTION_ALIAS, NULL},
-  {"file", ARG_FILE, N_("FILE"), 0, N_("Use FILE as the source message")},
+  {"file", ARG_FILE, N_("FILE"), 0, N_("use FILE as the source message")},
   {"license", ARG_LICENSE, 0,      0,
-   N_("Display software license"), -1},
+   N_("display software license"), -1},
   { 0 }
 };
 
@@ -73,7 +73,7 @@ add_folder (const char *folder)
 {
   if (!folder_name_list && mu_list_create (&folder_name_list))
     {
-      mu_error (_("Cannot create folder list"));
+      mu_error (_("cannot create folder list"));
       exit (1);
     }
   mu_list_append (folder_name_list, strdup (folder));
@@ -82,23 +82,24 @@ add_folder (const char *folder)
 void
 open_folders ()
 {
+  int rc;
   mu_iterator_t itr;
 
   if (!folder_name_list)
     {
-      mu_error (_("No folder specified"));
+      mu_error (_("no folder specified"));
       exit (1);
     }
 
-  if (mu_list_create (&folder_mbox_list))
+  if ((rc = mu_list_create (&folder_mbox_list)) != 0)
     {
-      mu_error (_("Cannot create folder list"));
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_list_create", NULL, rc);
       exit (1);
     }
 
-  if (mu_list_get_iterator (folder_name_list, &itr))
+  if ((rc = mu_list_get_iterator (folder_name_list, &itr)) != 0)
     {
-      mu_error (_("Cannot create iterator"));
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_list_get_iterator", NULL, rc);
       exit (1);
     }
 
@@ -123,7 +124,7 @@ enumerate_folders (void (*f) (void *, mu_mailbox_t), void *data)
 
   if (mu_list_get_iterator (folder_mbox_list, &itr))
     {
-      mu_error (_("Cannot create iterator"));
+      mu_error (_("cannot create iterator"));
       exit (1);
     }
 
@@ -150,7 +151,7 @@ close_folders ()
 }
 
 static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+opt_handler (int key, char *arg, struct argp_state *state)
 {
   switch (key)
     {
@@ -167,6 +168,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     case ARG_PRESERVE:
+      mh_opt_notimpl_warning ("-preserve");
       preserve_flag = is_true(arg);
       break;
 	
@@ -183,7 +185,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
 
     default:
-      return 1;
+      return ARGP_ERR_UNKNOWN;
     }
   return 0;
 }
@@ -197,7 +199,7 @@ refile_folder (void *data, mu_mailbox_t mbox)
   rc = mu_mailbox_append_message (mbox, msg);
   if (rc)
     {
-      mu_error (_("Error appending message: %s"), mu_strerror (rc));
+      mu_error (_("error appending message: %s"), mu_strerror (rc));
       exit (1);
     }
 }
@@ -229,9 +231,9 @@ main (int argc, char **argv)
   int status, i, j;
 
   /* Native Language Support */
-  mu_init_nls ();
+  MU_APP_INIT_NLS ();
 
-  mu_argp_init (program_version, NULL);
+  mh_argp_init (program_version);
   mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, &index);
 
@@ -257,7 +259,7 @@ main (int argc, char **argv)
       
       if (argc > 0)
 	{
-	  mu_error (_("Both message set and source file given"));
+	  mu_error (_("both message set and source file given"));
 	  exit (1);
 	}
       msg = mh_file_to_message (mu_folder_directory (), source_file);
