@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002, 2003, 2005, 2006, 2007, 
-   2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003, 2005, 2006, 2007, 2008, 2009, 2010 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -138,8 +138,8 @@ decode_cc_flag (const char *opt, const char *arg)
   return rc;
 }
 
-static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+static error_t
+opt_handler (int key, char *arg, struct argp_state *state)
 {
   char *s;
   
@@ -256,11 +256,14 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
 	  
     case ARG_INPLACE:
+      mh_opt_notimpl_warning ("-inplace");
+      break;
+      
     case ARG_WHATNOWPROC:
     case ARG_NOWHATNOWPROC:
-      argp_error (state, _("Option is not yet implemented"));
-      exit (1);
-      
+      mh_opt_notimpl ("-[no]whatnowproc");
+      break;
+
     case ARG_LICENSE:
       mh_license (argp_program_version);
       break;
@@ -271,7 +274,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     default:
-      return 1;
+      return ARGP_ERR_UNKNOWN;
     }
   return 0;
 }
