@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,48 +34,51 @@ static char args_doc[] = N_("file [file...]");
 /* GNU options */
 static struct argp_option options[] = {
   {"alias",         ARG_ALIAS,         N_("FILE"), 0,
-   N_("Specify additional alias file") },
+   N_("specify additional alias file") },
   {"draft",         ARG_DRAFT,         NULL, 0,
-   N_("Use prepared draft") },
+   N_("use prepared draft") },
   {"draftfolder",   ARG_DRAFTFOLDER,   N_("FOLDER"), 0,
-   N_("Specify the folder for message drafts") },
+   N_("specify the folder for message drafts") },
   {"draftmessage",  ARG_DRAFTMESSAGE,  NULL, 0,
-   N_("Treat the arguments as a list of messages from the draftfolder") },
+   N_("treat the arguments as a list of messages from the draftfolder") },
   {"nodraftfolder", ARG_NODRAFTFOLDER, NULL, 0,
-   N_("Undo the effect of the last --draftfolder option") },
+   N_("undo the effect of the last --draftfolder option") },
   {"filter",        ARG_FILTER,        N_("FILE"), 0,
-  N_("* Use filter FILE to preprocess the body of the message") },
+  N_("* use filter FILE to preprocess the body of the message") },
   {"nofilter",      ARG_NOFILTER,      NULL, 0,
-   N_("* Undo the effect of the last --filter option") },
+   N_("* undo the effect of the last --filter option") },
   {"format",        ARG_FORMAT,        N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("* Reformat To: and Cc: addresses") },
+   N_("* reformat To: and Cc: addresses") },
   {"noformat",      ARG_NOFORMAT,      NULL, OPTION_HIDDEN, "" },
   {"forward",       ARG_FORWARD,       N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("* In case of failure forward the draft along with the failure notice to the sender") },
+   N_("* in case of failure forward the draft along with the failure notice to the sender") },
   {"noforward",     ARG_NOFORWARD,     NULL, OPTION_HIDDEN, "" },
   {"mime",          ARG_MIME,          N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("* Use MIME encapsulation") },
+   N_("* use MIME encapsulation") },
   {"nomime",        ARG_NOMIME,        NULL, OPTION_HIDDEN, "" },
   {"msgid",         ARG_MSGID,         N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Add Message-ID: field") },
+   N_("add Message-ID: field") },
   {"nomsgid",       ARG_NOMSGID,       NULL, OPTION_HIDDEN, ""},
   {"push",          ARG_PUSH,          N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Run in the backround.") },
+   N_("run in the backround") },
   {"nopush",        ARG_NOPUSH,        NULL, OPTION_HIDDEN, "" },
+  {"preserve",      ARG_PRESERVE,      N_("BOOL"), OPTION_ARG_OPTIONAL,
+   N_("keep draft files") },
+  {"keep",          0, NULL, OPTION_ALIAS, NULL},
   {"split",         ARG_SPLIT,         N_("SECONDS"), 0,
-   N_("Split the draft into several partial messages and send them with SECONDS interval") },
+   N_("split the draft into several partial messages and send them with SECONDS interval") },
   {"chunksize",     ARG_CHUNKSIZE,     N_("NUMBER"), 0,
-   N_("Set the size of chunk for --split (in bytes)") },
+   N_("set the size of chunk for --split (in bytes)") },
   {"verbose",       ARG_VERBOSE,       N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Print the transcript of interactions with the transport system") },
+   N_("print the transcript of interactions with the transport system") },
   {"noverbose",     ARG_NOVERBOSE,     NULL, OPTION_HIDDEN, "" },
   {"watch",         ARG_WATCH,         N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("Monitor the delivery of mail") },
+   N_("monitor the delivery of mail") },
   {"nowatch",       ARG_NOWATCH,       NULL, OPTION_HIDDEN, "" },
   {"width",         ARG_WIDTH,         N_("NUMBER"), 0,
-   N_("* Make header fields no longer than NUMBER columns") },
+   N_("* make header fields no longer than NUMBER columns") },
   {"license", ARG_LICENSE, 0,      0,
-   N_("Display software license"), -1},
+   N_("display software license"), -1},
   { 0 }
 };
 
@@ -91,7 +95,9 @@ struct mh_option mh_option[] = {
   {"forward",       4, MH_OPT_BOOL, NULL},
   {"mime",          2, MH_OPT_BOOL, NULL},
   {"msgid",         2, MH_OPT_BOOL, NULL},
-  {"push",          1, MH_OPT_BOOL, NULL},
+  {"push",          2, MH_OPT_BOOL, NULL},
+  {"preserve",      2, MH_OPT_BOOL, NULL},
+  {"keep",          1, MH_OPT_BOOL, NULL},
   {"split",         1, 0, "seconds"},
   {"verbose",       1, MH_OPT_BOOL, NULL},
   {"watch",         2, MH_OPT_BOOL, NULL},
@@ -116,6 +122,8 @@ static int verbose;              /* Produce verbose diagnostics */
 static int watch;                /* Watch the delivery process */
 static unsigned width = 76;      /* Maximum width of header fields */
 
+static int keep_files;           /* Keep draft files */
+
 #define DEFAULT_X_MAILER "MH (" PACKAGE_STRING ")"
 
 #define WATCH(c) do {\
@@ -123,8 +131,8 @@ static unsigned width = 76;      /* Maximum width of header fields */
     watch_printf c;\
 } while (0)
 
-static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+static error_t
+opt_handler (int key, char *arg, struct argp_state *state)
 {
   char *p;
   
@@ -138,7 +146,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       split_size = strtoul (arg, &p, 10);
       if (*p)
 	{
-	  argp_error (state, "%s: %s", arg, _("Invalid number"));
+	  argp_error (state, "%s: %s", arg, _("invalid number"));
 	  exit (1);
 	}
       break;
@@ -162,30 +170,40 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     case ARG_FILTER:
+      mh_opt_notimpl ("-filter");
+      break;
+
     case ARG_NOFILTER:
-      return 1;
-      
+      mh_opt_notimpl ("-nofilter");
+      break;
+ 
     case ARG_FORMAT:
+      mh_opt_notimpl_warning ("-format"); 
       reformat_recipients = is_true (arg);
       break;
       
     case ARG_NOFORMAT:
+      mh_opt_notimpl_warning ("-noformat"); 
       reformat_recipients = 0;
       break;
       
     case ARG_FORWARD:
+      mh_opt_notimpl_warning ("-forward");
       forward_notice = is_true (arg);
       break;
       
     case ARG_NOFORWARD:
+      mh_opt_notimpl_warning ("-noforward");
       forward_notice = 0;
       break;
       
     case ARG_MIME:
+      mh_opt_notimpl_warning ("-mime");
       mime_encaps = is_true (arg);
       break;
       
     case ARG_NOMIME:
+      mh_opt_notimpl_warning ("-nomime");
       mime_encaps = 0;
       break;
       
@@ -196,6 +214,10 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
     case ARG_NOMSGID:
       append_msgid = 0;
       break;
+
+    case ARG_PRESERVE:
+      keep_files = is_true (arg);
+      break;
       
     case ARG_PUSH:
       background = is_true (arg);
@@ -210,7 +232,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       split_interval = strtoul (arg, &p, 10);
       if (*p)
 	{
-	  argp_error (state, "%s: %s", arg, _("Invalid number"));
+	  argp_error (state, "%s: %s", arg, _("invalid number"));
 	  exit (1);
 	}
       break;
@@ -232,10 +254,11 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     case ARG_WIDTH:
+      mh_opt_notimpl_warning ("-width");
       width = strtoul (arg, &p, 10);
       if (*p)
 	{
-	  argp_error (state, _("Invalid number"));
+	  argp_error (state, _("invalid number"));
 	  exit (1);
 	}
       break;
@@ -245,7 +268,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
 
     default:
-      return 1;
+      return ARGP_ERR_UNKNOWN;
     }
   return 0;
 }
@@ -287,7 +310,7 @@ check_file (char *name)
   if (!mesg_list && mu_list_create (&mesg_list))
     {
       free (file_name);
-      mu_error (_("Cannot create message list"));
+      mu_error (_("cannot create message list"));
       return 1;
     }
   elt = xmalloc (sizeof *elt);
@@ -320,7 +343,7 @@ read_mts_profile ()
       mu_set_user_email_domain (p);
     }
   else if ((rc = mu_get_host_name (&hostname)))
-    mu_error (_("Cannot get system host name: %s"), mu_strerror (rc));
+    mu_error (_("cannot get system host name: %s"), mu_strerror (rc));
 
   if ((p = mh_context_get_value (mts_profile, "localdomain", NULL)))
     {
@@ -337,7 +360,7 @@ read_mts_profile ()
       free (newdomain);
       if (rc)
 	{
-	  mu_error (_("Cannot set user mail domain: %s"), mu_strerror (rc));
+	  mu_error (_("cannot set user mail domain: %s"), mu_strerror (rc));
 	  exit (1);
 	}
     }
@@ -352,7 +375,7 @@ read_mts_profile ()
       rc = mu_get_user_email_domain (&domain);
       if (rc)
 	{
-	  mu_error (_("Cannot get user email: %s"), mu_strerror (rc));
+	  mu_error (_("cannot get user email: %s"), mu_strerror (rc));
 	  exit (1);
 	}
       len = strlen (p) + 1 + strlen (domain) + 1;
@@ -364,7 +387,7 @@ read_mts_profile ()
       rc = mu_set_user_email (newemail);
       if (rc)
 	{
-	  mu_error (_("Cannot set user email (%s): %s"),
+	  mu_error (_("cannot set user email (%s): %s"),
 		    newemail, mu_strerror (rc));
 	  exit (1);
 	}
@@ -387,7 +410,7 @@ open_mailer ()
   status = mu_mailer_create (&mailer, url);
   if (status)
     {
-      mu_error (_("Cannot create mailer `%s'"), url);
+      mu_error (_("cannot create mailer `%s'"), url);
       return NULL;
     }
 
@@ -402,7 +425,7 @@ open_mailer ()
   status = mu_mailer_open (mailer, MU_STREAM_RDWR);
   if (status)
     {
-      mu_error (_("Cannot open mailer `%s'"), url);
+      mu_error (_("cannot open mailer `%s'"), url);
       return NULL;
     }
   return mailer;
@@ -530,7 +553,7 @@ fix_fcc (mu_message_t msg)
       if (need_fixup)
 	{
 	  mu_header_set_value (hdr, MU_HEADER_FCC, fcc, 1);
-	  WATCH ((_("fixed fcc: %s"), fcc));
+	  WATCH ((_("Fixed fcc: %s"), fcc));
 	}
       free (fcc);
     }	  
@@ -572,7 +595,7 @@ fix_dcc (mu_message_t msg)
       else
 	bcc = dcc;
 
-      WATCH ((_("fixed bcc: %s"), bcc));
+      WATCH ((_("Fixed bcc: %s"), bcc));
       mu_header_set_value (hdr, MU_HEADER_BCC, bcc, 1);
       free (bcc);
     }
@@ -595,12 +618,12 @@ backup_file (const char *file_name)
       new_name[0] = ',';
       strcpy (new_name + 1, file_name);
     }
-  WATCH ((_("renaming %s to %s"), file_name, new_name));
+  WATCH ((_("Renaming %s to %s"), file_name, new_name));
 
   if (unlink (new_name) && errno != ENOENT)
-    mu_error (_("Cannot unlink file `%s': %s"), new_name, mu_strerror (errno));
+    mu_diag_funcall (MU_DIAG_ERROR, "unlink", new_name, errno);
   else if (rename (file_name, new_name))
-    mu_error (_("Cannot rename `%s' to `%s': %s"),
+    mu_error (_("cannot rename `%s' to `%s': %s"),
 	      file_name, new_name, mu_strerror (errno));
   free (new_name);
 }
@@ -682,7 +705,7 @@ _action_send (void *item, void *data)
     rc = mu_mailer_send_message (mailer, msg, NULL, NULL);
   if (rc)
     {
-      mu_error(_("Cannot send message: %s"), mu_strerror (rc));
+      mu_error(_("cannot send message: %s"), mu_strerror (rc));
       return 1;
     }
 
@@ -690,7 +713,8 @@ _action_send (void *item, void *data)
   mu_mailer_close (mailer);
   mu_mailer_destroy (&mailer);
 
-  backup_file (elt->file_name);
+  if (!keep_files)
+    backup_file (elt->file_name);
   
   return 0;
 }
@@ -712,7 +736,7 @@ do_send (int argc, char **argv)
   
   if (background && daemon (0, 0) < 0)
     {
-      mu_error(_("Cannot switch to background: %s"), mu_strerror (errno));
+      mu_error (_("cannot switch to background: %s"), mu_strerror (errno));
       return 1;
     }
 
@@ -751,7 +775,7 @@ main (int argc, char **argv)
 
       if (stat (xargv[0], &st))
 	{
-	  mu_error(_("Cannot stat %s: %s"), xargv[0], mu_strerror (errno));
+	  mu_diag_funcall (MU_DIAG_ERROR, "stat", xargv[0], errno);
 	  return 1;
 	}
 
