@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2007, 2008, 2009, 2010 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,18 +44,18 @@ N_("\n\nDebug flags are:\n\
 
 static struct argp_option options[] = {
   {"no-ask", 'a', N_("TYPE-LIST"), OPTION_ARG_OPTIONAL,
-   N_("Do not ask for confirmation before displaying files. If TYPE-LIST is given, do not ask for confirmation before displaying such files whose MIME type matches one of the patterns from TYPE-LIST"), 0},
+   N_("do not ask for confirmation before displaying files, or, if TYPE-LIST is given, do not ask for confirmation before displaying such files whose MIME type matches one of the patterns from TYPE-LIST"), 0},
   {"no-interactive", 'h', NULL, 0,
-   N_("Disable interactive mode"), 0 },
+   N_("disable interactive mode"), 0 },
   {"print", 0, NULL, OPTION_ALIAS, NULL, 0 },
   {"debug",  'd', N_("FLAGS"),  OPTION_ARG_OPTIONAL,
-   N_("Enable debugging output"), 0},
+   N_("enable debugging output"), 0},
   {"mimetypes", 't', N_("FILE"), 0,
-   N_("Use this mime.types file"), 0},
+   N_("use this mime.types file"), 0},
   {"dry-run", 'n', NULL, 0,
-   N_("Do not do anything, just print what whould be done"), 0},
+   N_("do not do anything, just print what whould be done"), 0},
   {"metamail", OPT_METAMAIL, N_("FILE"), OPTION_ARG_OPTIONAL,
-   N_("Use metamail to display files"), 0},
+   N_("use metamail to display files"), 0},
   {0, 0, 0, 0}
 };
 
@@ -69,7 +70,7 @@ char *mimeview_file;       /* Name of the file to view */
 FILE *mimeview_fp;     /* Its descriptor */
 
 static void
-set_debug_flags (mu_debug_t debug, char *arg)
+set_debug_flags (mu_debug_t debug, const char *arg)
 {
   for (; *arg; arg++)
     {
@@ -92,7 +93,7 @@ set_debug_flags (mu_debug_t debug, char *arg)
 static error_t
 parse_opt (int key, char *arg, struct argp_state *state)
 {
-  static struct mu_argp_node_list lst;
+  static mu_list_t lst;
 
   switch (key)
     {
@@ -107,7 +108,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case ARGP_KEY_FINI:
       if (dry_run && !debug_level)
 	debug_level = 1;
-      mu_argp_node_list_finish (&lst, NULL, NULL);
+      mu_argp_node_list_finish (lst, NULL, NULL);
       break;
 
     case 'a':
@@ -116,7 +117,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
       
     case 'd':
-      mu_argp_node_list_new (&lst, "debug", arg ? arg : "9");
+      mu_argp_node_list_new (lst, "debug", arg ? arg : "9");
       break;
 
     case 'h':
@@ -128,11 +129,11 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
       
     case 't':
-      mu_argp_node_list_new (&lst, "mimetypes", arg);
+      mu_argp_node_list_new (lst, "mimetypes", arg);
       break;
 
     case OPT_METAMAIL:
-      mu_argp_node_list_new (&lst, "metamail", arg ? arg : "metamail");
+      mu_argp_node_list_new (lst, "metamail", arg ? arg : "metamail");
       break;
       
     default: 
@@ -152,16 +153,11 @@ static struct argp argp = {
 
 
 static int
-cb_debug (mu_debug_t debug, void *data, char *arg)
+cb_debug (mu_debug_t debug, void *data, mu_config_value_t *val)
 {
-  set_debug_flags (debug, arg);
-  return 0;
-}
-
-static int
-cb_metamail (mu_debug_t debug, void *data, char *arg)
-{
-  metamail = strdup ("metamail");
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+    return 1;
+  set_debug_flags (debug, val->v.string);
   return 0;
 }
 
@@ -172,7 +168,7 @@ struct mu_cfg_param mimeview_cfg_param[] = {
   { "mimetypes", mu_cfg_string, &mimetypes_config, 0, NULL,
     N_("Use this mime.types file."),
     N_("file") },
-  { "metamail", mu_cfg_string, NULL, 0, cb_metamail,
+  { "metamail", mu_cfg_string, &metamail, 0, NULL,
     N_("Use this program to display files."),
     N_("prog") },
   { NULL }
@@ -193,12 +189,12 @@ open_file (char *name)
   struct stat st;
   if (stat (name, &st))
     {
-      mu_error (_("Cannot stat `%s': %s"), name, mu_strerror (errno));
+      mu_error (_("cannot stat `%s': %s"), name, mu_strerror (errno));
       return -1;
     }
   if (!S_ISREG (st.st_mode) && !S_ISLNK (st.st_mode))
     {
-      mu_error (_("Not a regular file or symbolic link: `%s'"), name);
+      mu_error (_("not a regular file or symbolic link: `%s'"), name);
       return -1;
     }
 
@@ -257,7 +253,7 @@ display_file (const char *type)
       asprintf (&text, "Content-Type: %s\n", type);
       status = mu_header_create (&hdr, text, strlen (text), NULL);
       if (status)
-	mu_error (_("Cannot create header: %s"), mu_strerror (status));
+	mu_error (_("cannot create header: %s"), mu_strerror (status));
       else
 	{
 	  mu_stdio_stream_create (&stream, mimeview_fp,
@@ -292,7 +288,7 @@ main (int argc, char **argv)
 
   if (argc == 0)
     {
-      mu_error (_("No files given"));
+      mu_error (_("no files given"));
       return 1;
     }
 
