@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005, 2007, 2009, 2010 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -21,7 +22,7 @@
 /* EXPIRE see RFC2449:
 
   Implementation:
-    When a message is downloaded by RETR, it is marked with
+    When a message is downloaded by RETR or TOP, it is marked with
     "X-Expire-Timestamp: N", where N is the current value of
     UNIX timestamp.
 
@@ -67,7 +68,7 @@ header_is_expired (mu_header_t hdr)
 			buf, sizeof buf, NULL))
     return 0;
   timestamp = strtoul (buf, &p, 0);
-  while (*p && isspace (*p))
+  while (*p && mu_isspace (*p))
     p++;
   if (*p)
     return 0;
