@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 
-   2004, 2005, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008,
+   2009, 2010 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,20 +34,20 @@ static void print_body (mu_message_t);
 static int  string_starts_with (const char * s1, const char *s2);
 
 const char *program_version = "readmsg (" PACKAGE_STRING ")";
-static char doc[] = N_("GNU readmsg -- print messages");
+static char doc[] = N_("GNU readmsg -- print messages.");
 static error_t readmsg_parse_opt  (int key, char *arg, struct argp_state *astate);
 
 static struct argp_option options[] = 
 {
-  { "debug", 'd', 0, 0, N_("Display debugging information"), 1 },
-  { "header", 'h', 0, 0, N_("Display entire header"), 1 },
+  { "debug", 'd', 0, 0, N_("display debugging information"), 1 },
+  { "header", 'h', 0, 0, N_("display entire header"), 1 },
   { "weedlist", 'w', N_("LIST"), 0,
-    N_("List of header names separated by whitespace or commas"), 1 },
-  { "folder", 'f', N_("FOLDER"), 0, N_("Folder to use"), 1 },
-  { "no-header", 'n', 0, 0, N_("Exclude all headers"), 1 },
-  { "form-feeds", 'p', 0, 0, N_("Output formfeeds between messages"), 1 },
+    N_("list of header names separated by whitespace or commas"), 1 },
+  { "folder", 'f', N_("FOLDER"), 0, N_("folder to use"), 1 },
+  { "no-header", 'n', 0, 0, N_("exclude all headers"), 1 },
+  { "form-feeds", 'p', 0, 0, N_("output formfeeds between messages"), 1 },
   { "show-all-match", 'a', NULL, 0,
-    N_("Print all messages matching pattern, not only the first"), 1 },
+    N_("print all messages matching pattern, not only the first"), 1 },
   {0, 0, 0, 0}
 };
 
@@ -79,7 +79,7 @@ int show_all = 0;
 static error_t
 readmsg_parse_opt (int key, char *arg, struct argp_state *astate)
 {
-  static struct mu_argp_node_list lst;
+  static mu_list_t lst;
 
   switch (key)
     {
@@ -88,27 +88,27 @@ readmsg_parse_opt (int key, char *arg, struct argp_state *astate)
       break;
 
     case 'h':
-      mu_argp_node_list_new (&lst, "header", "yes");
+      mu_argp_node_list_new (lst, "header", "yes");
       break;
 
     case 'f':
-      mu_argp_node_list_new (&lst, "folder", arg);
+      mu_argp_node_list_new (lst, "folder", arg);
       break;
 
     case 'w':
-      mu_argp_node_list_new (&lst, "weedlist", arg);
+      mu_argp_node_list_new (lst, "weedlist", arg);
       break;
 
     case 'n':
-      mu_argp_node_list_new (&lst, "no-header", "yes");
+      mu_argp_node_list_new (lst, "no-header", "yes");
       break;
 
     case 'p':
-      mu_argp_node_list_new (&lst, "form-feeds", arg);
+      mu_argp_node_list_new (lst, "form-feeds", "yes");
       break;
 	  
     case 'a':
-      mu_argp_node_list_new (&lst, "show-all-match", arg);
+      mu_argp_node_list_new (lst, "show-all-match", "yes");
       break;
 
     case ARGP_KEY_INIT:
@@ -117,8 +117,8 @@ readmsg_parse_opt (int key, char *arg, struct argp_state *astate)
       
     case ARGP_KEY_FINI:
       if (dbug)
-	mu_argp_node_list_new (&lst, "debug", mu_umaxtostr (0, dbug));
-      mu_argp_node_list_finish (&lst, NULL, NULL);
+	mu_argp_node_list_new (lst, "debug", mu_umaxtostr (0, dbug));
+      mu_argp_node_list_finish (lst, NULL, NULL);
       break;
       
     default:
@@ -216,7 +216,7 @@ print_header (mu_message_t message, int unix_header, int weedc, char **weedv)
       while (mu_stream_read (stream, buf, sizeof (buf) - 1, offset, &len) == 0
 	     && len != 0)
 	{
-	  buf[len] ='\0';
+	  buf[len] = '\0';
 	  printf ("%s", buf);
 	  offset += len;
 	}
@@ -271,17 +271,12 @@ print_body (mu_message_t message)
   while (mu_stream_read (stream, buf, sizeof (buf) - 1, offset, &len) == 0
 	 && len != 0)
     {
-      buf[len] ='\0';
+      buf[len] = '\0';
       printf ("%s", buf);
       offset += len;
     }
 }
 
-/* This is still work in progress  */
-/* FIXME: Parse options:  See readmsg(1) part of elm:
-   readmsg 1 3 0
-   extracts three messages from the folder: the first, the third, and
-   the last. */
 int
 main (int argc, char **argv)
 {
@@ -314,12 +309,12 @@ main (int argc, char **argv)
   if (status != 0)
     {
       if (mailbox_name)
-	fprintf (stderr, _("Could not create mailbox `%s': %s\n"),
-		 mailbox_name,
-		 mu_strerror(status));
+	mu_error (_("could not create mailbox `%s': %s"),
+		  mailbox_name,
+		  mu_strerror(status));
       else
-	fprintf (stderr, _("Could not create default mailbox: %s\n"),
-		 mu_strerror(status));
+	mu_error (_("could not create default mailbox: %s"),
+		  mu_strerror(status));
       exit (2);
     }
 
@@ -337,7 +332,7 @@ main (int argc, char **argv)
       mu_url_t url = NULL;
 
       mu_mailbox_get_url (mbox, &url);
-      mu_error (_("Could not open mailbox `%s': %s\n"),
+      mu_error (_("could not open mailbox `%s': %s"),
 		mu_url_to_string (url), mu_strerror (status));
       exit (2);
     }
@@ -349,13 +344,13 @@ main (int argc, char **argv)
 			 &weedc, &weedv);
   if (status)
     {
-      mu_error (_("Cannot parse weedlist: %s"), mu_strerror (status));
+      mu_error (_("cannot parse weedlist: %s"), mu_strerror (status));
       exit (2);
     }
 
   for (i = 0; i < weedc; i++)
     {
-      if (strcasecmp (weedv[i], "From_") == 0)
+      if (mu_c_strcasecmp (weedv[i], "From_") == 0)
 	{
 	  int j;
 	  unix_header = 1;
