@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005, 2006, 2007, 2009, 2010 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -19,7 +20,7 @@
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
-#include <ctype.h>
+#include <mailutils/cctype.h>
 #include <mailutils/mailutils.h>
 #include <mailutils/sql.h>
 
@@ -93,7 +94,7 @@ mu_mysql_connect (mu_sql_connection_t conn)
   
   mysql_init (mp->mysql);
 
-  if (conn->server[0] == '/')
+  if (conn->server && conn->server[0] == '/')
     {
       host = "localhost";
       socket_name = conn->server;
@@ -108,7 +109,7 @@ mu_mysql_connect (mu_sql_connection_t conn)
 			  conn->dbname,
 			  conn->port,
 			  socket_name,
-			  0))
+			  CLIENT_MULTI_RESULTS))
     return MU_ERR_SQL;
   
   return 0;
@@ -340,7 +341,7 @@ mu_check_mysql_scrambled_password (const char *scrambled, const char *message)
 
   /* Try to normalize it by cutting off trailing whitespace */
   for (p = scrambled + strlen (scrambled) - 1;
-       p > scrambled && isspace (*p); p--)
+       p > scrambled && mu_isspace (*p); p--)
     ;
   switch (p - scrambled)
     {
